<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://4dealer.ru/
 * @since      1.0.0
 *
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/admin
 */
/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/admin
 * @author     Maksim Onokhin <root@cybermax.spb.ru>
 */
if (!defined('WP_FDEAL_LOGGER')) {
    define('WP_FDEAL_LOGGER', true);
}

register_shutdown_function('kill_pid');

function kill_pid()
{
    if (isset($_SESSION['4D_PID']) && count($_SESSION['4D_PID']) > 0) {
        foreach ($_SESSION['4D_PID'] as $i => $pid) {
            unlink(WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/' . $pid);
            unset($_SESSION['4D_PID'][$i]);
        }
    }
}

class Wp_Fdeal_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    private $cache = [];

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;
    private $db;

    private $MAX_PROCESSES_PER_JOBS;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     * @since    1.0.0
     */
    public function __construct($plugin_name, $version,$actualVersion)
    {
        global $wpdb;
        $this->db = $wpdb;
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->actualVersion = $actualVersion;
        ini_set('default_socket_timeout', 600);

        $this->defaultPidMethod = true; // Use PHP PID file
        $this->defaultPidMethod = false; // Use self PID

        $this->MAX_PROCESSES_PER_JOBS = (int) get_option("{$this->plugin_name}-4dealer_calls_per_iteration", 1);

        if (!is_numeric($this->MAX_PROCESSES_PER_JOBS) || $this->MAX_PROCESSES_PER_JOBS <= 0) {
            $this->MAX_PROCESSES_PER_JOBS = 1;
        }

        require_once(ABSPATH . "wp-admin" . '/includes/image.php');
        require_once(ABSPATH . "wp-admin" . '/includes/file.php');
        require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Wp_Fdeal_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Wp_Fdeal_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/wp-fdeal-admin.css', array(), $this->version, 'all');
        wp_enqueue_style($this->plugin_name . '-notify', plugin_dir_url(__FILE__) . 'css/jquery.notify.css', array(), $this->version, 'all');
        wp_enqueue_style($this->plugin_name . '-bootstrap', plugin_dir_url(__FILE__) . 'css/bootstrap.css', array(), $this->version, 'all');
        wp_enqueue_style($this->plugin_name . '-jquery-ui', plugin_dir_url(__FILE__) . 'css/jquery-ui.min.css', array(), $this->version, 'all');
    }



    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Wp_Fdeal_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Wp_Fdeal_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script($this->plugin_name . '-bootstrap', plugin_dir_url(__FILE__) . 'js/bootstrap.bundle.min.js', array('jquery'), $this->version, true);
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/wp-fdeal-admin.js', array('jquery'), $this->version, false);
        wp_enqueue_script($this->plugin_name . '-jquery-ui', plugin_dir_url(__FILE__) . 'js/jquery-ui.min.js', array('jquery'), $this->version, true);
        wp_enqueue_script($this->plugin_name . '-notify', plugin_dir_url(__FILE__) . 'js/jquery.notify.min.js', array('jquery'), $this->version, true);
    }

    /**
     *
     * admin/class-wp-cbf-admin.php - Don't add this
     *
     * */

    /**
     * Register the administration menu for this plugin into the WordPress Dashboard menu.
     *
     * @since    1.0.0
     */
    public function add_plugin_admin_menu()
    {

        /*
         * Add a settings page for this plugin to the Settings menu.
         *
         * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
         *
         *        Administration Menus: http://codex.wordpress.org/Administration_Menus
         *
         */
        add_options_page('Настройки 4dealer импорт', '4dealer Импорт', 'manage_options', $this->plugin_name, array($this, 'display_plugin_setup_page'));
        add_menu_page('4dealer Импорт', '4dealer', 'read', '4dealer_management', array($this, 'display_management_page'));
        add_action('admin_init', array($this, 'register_settings'));
        //add_action( 'woocommerce_product_options_general_product_data', array($this,'woo_add_custom_general_fields') );
        //add_action( 'woocommerce_process_product_meta', array($this,'woo_add_custom_general_fields_save') );
    }

    function get_dealer_categories()
    {
        $dealer_id = isset($_REQUEST['id']) ? (int)$_REQUEST['id'] : false;
        if (!$dealer_id) {
            return false;
        }
        echo wp_json_encode($this->getLocalDealerCategories($dealer_id));
    }

    public function register_settings()
    {
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_login");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_token");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_hide_duplicates");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_hide_duplicates_rule");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_hide_goods_without_pics");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_short_step");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_full_step");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_calls_per_iteration");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_exclude_str");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_logger");
        register_setting($this->plugin_name . '-settings-group', $this->plugin_name . "-4dealer_pid_method");
    }

    /**
     * Add settings action link to the plugins page.
     *
     * @since    1.0.0
     */
    public function add_action_links($links)
    {
        /*
         *  Documentation : https://codex.wordpress.org/Plugin_API/Filter_Reference/plugin_action_links_(plugin_file_name)
         */
        $settings_link = array('<a href="' . admin_url('options-general.php?page=' . $this->plugin_name) . '">' . __('Настройки', $this->plugin_name) . '</a>',);
        return array_merge($settings_link, $links);
    }

    /**
     * Render the settings page for this plugin.
     *
     * @since    1.0.0
     */
    public function display_plugin_setup_page()
    {
        include_once("partials/{$this->plugin_name}-admin-display.php");
    }

    public function display_management_page()
    {
        $active_tab = isset($_GET['tab']) ? $_GET['tab'] : 'dealers_list';
        $method = 'prepare_' . $active_tab;
        if (method_exists($this, $method)) {
            $data = $this->$method();
        }
        //$prices=$this->getLocalPriceList();
        //if(!$prices){
        $prices = $this->getRemotePriceList();
        //}
        include_once("partials/{$this->plugin_name}-management-admin-display.php");
    }

    public function prepare_dealer_detail()
    {
        $data = array('prices' => $this->getLocalPriceList());
        $data['selected_id'] = 0;

        if (isset($_REQUEST['dealer_id'])) {

            $data['selected_id'] = (int)$_REQUEST['dealer_id'];
            $data['dealer'] = $this->get_price_info_by_id($data['selected_id']);
            $data['price_markups'] = $this->get_price_markups_by_dealer_id($data['selected_id']);
        }

        return $data;
    }

    public function prepare_options_merge()
    {
        $data = array('prices' => $this->getLocalPriceList());
        $data['selected_id'] = 0;

        $data['categoriesTreeExtraData'] = [
            'countInSection' => []
        ];

        $data['checkboxAutoLoadOptions'] = false;
        $data['checkboxPublicOptions'] = false;

        if (isset($_REQUEST['dealer_id']) && is_array($data['prices'])) {
            $data['selected_id'] = (int)$_REQUEST['dealer_id'];
            $data['section_id'] = isset($_REQUEST['section_id'])?(int)$_REQUEST['section_id']:0;
            $data['options'] = $this->get_options_merge($data['selected_id'], false, ['section_id' => $data['section_id']]);

            foreach ($data['prices'] as $price) {
                if ($price->price_id == $data['selected_id']) {
                    $data['checkboxAutoLoadOptions'] = $price->auto_load_options;
                    $data['checkboxPublicOptions'] = $price->public_options;
                }
            }
        }

        if (isset($_REQUEST['dealer_id'])) {
            $data['local_merged_categories'] = $this->get_categories_terms_list($data['selected_id']);
        }

        /**
         * Подсчитываем количество в существующих options_merge
         */
        $countOptionsBySections = $this->db->get_results(
            "SELECT section_id, COUNT(id) AS cnt FROM `{$this->db->prefix}4dealer_options_merge` GROUP BY section_id",
            ARRAY_A
        );

        foreach ($countOptionsBySections as $count) {
            $data['categoriesTreeExtraData']['countInSection'][$count['section_id']] = $count['cnt'];
        }

        $data['categoriesTree'] = $this->getLocalDealerCategories($data['selected_id']);

        return $data;
    }

    public function prepare_dealers_list()
    {
        $prices = $this->getLocalPriceList();

        if (!$prices) {
            $prices = $this->getRemotePriceList();
        }

        return array('prices' => $prices, 'jobs' => $this->get_jobs_queue());
    }

    public function prepare_cat_map()
    {
        $product_categories = get_terms('product_cat', ['hide_empty' => 0]);

        $this->sortArrayLocalCategoriesTree($product_categories);

        $data = [
            'prices' => $this->getRemotePriceList(),
            'local_categories' => $product_categories,
            'local_categories_tree' => $this->buildTree($product_categories),
            'selected_id' => 0
        ];

        if (isset($_REQUEST['dealer_id'])) {
            $data['selected_id'] = (int)$_REQUEST['dealer_id'];
            $data['dealer_categories'] = $this->getLocalDealerCategories($data['selected_id']);
            $flat_categories_list = array();
            $this->buildFlatCategoriesList($data['dealer_categories'], $flat_categories_list);
            $data['dealer_categories_flat'] = $flat_categories_list;
            $data['mappings'] = $this->get_categories_mapping_by_price_id($data['selected_id']);
        }

        return $data;
    }

    function sortArrayDealerCategoriesTree(&$dealer_categories)
    {
        if ($dealer_categories == false) {
            return;
        }
        usort($dealer_categories, function($a, $b) {
            return mb_strcmp($a['name'], $b['name']);
        });

        foreach ($dealer_categories as $key => $dealerCategory) {
            if (isset($dealerCategory['children']) && is_array($dealerCategory['children'])) {
                $this->sortArrayDealerCategoriesTree($dealerCategory['children']);
            }

            $dealer_categories[$key] = $dealerCategory;
        }
    }

    function sortArrayLocalCategoriesTree(&$local_categories)
    {
        usort($local_categories, function($a, $b) {
            return mb_strcmp($a->name, $b->name);
        });
    }

    public function get_price_markups_by_dealer_id($id)
    {
        $get_markups = $this->db->get_results("select * from {$this->db->prefix}4dealer_price_markup where price_id=$id", ARRAY_A);
        return is_null($get_markups) ? false : $get_markups;
    }

    public function get_price_markups_by_dealer_categories($id, $cat_ids, $cache = false)
    {
        if ($cache && $this->get_markups_by_dealer_categories[$id . $cat_ids]) {
            return $this->get_markups_by_dealer_categories[$id . $cat_ids];
        }

        $get_markup = $this->db->get_results("select * from {$this->db->prefix}4dealer_price_markup where price_id=$id and category_id IN ($cat_ids)", ARRAY_A);
        $res = is_null($get_markup) ? false : $get_markup;

        return $this->get_markups_by_dealer_categories[$id . $cat_ids] = $res;
    }

    public function get_categorized_price_markups($id)
    {
        $get_markups = $this->get_price_markups_by_dealer_id((int)$id);
        if ($get_markups) {
            foreach ($get_markups as $markup) {
                $markups[$markup['category_id']][] = $markup;
            }
            return $markups;
        }
        return array();
    }

    private function buildTree(&$elements, $parentId = 0, $parents_info = false)
    {
        $branch = array();
        foreach ($elements as $key => $element) {
            if ($element->parent != $parentId) {
                continue;
            }

            $children = self::buildTree($elements, $element->term_id);
            if (!empty($children)) {
                $element->children = $children;
            }
            unset($children);
            if ($parents_info) {
                $element->parents_array[] = $parentId;
            }
            $branch[$element->term_id] = $element;
        }
        unset($key, $element);
        return $branch;
    }

    function buildFlatCategoriesList($categories, &$categories_list)
    {
        if (!is_array($categories)) $categories = [];

        foreach ($categories as $category) {
            $categories_list[$category['id']] = $category;
            if (isset($category['children'])) {
                unset($categories_list[$category['id']]['children']);
                $this->buildFlatCategoriesList($category['children'], $categories_list);
            }
        }

        return $categories_list;
    }

    public function getLocalPriceList()
    {
        $get_prices = $this->db->get_results("select * from {$this->db->prefix}4dealer_prices");
        if ($this->db->num_rows == 0) {
            return false;
        }
        $prices = array();
        foreach ($get_prices as $price) {
            $prices[] = $price;
        }

        return $prices;
    }

    public function getLocalDealerCategories($dealer_id)
    {
        $categories = $this->db->get_var("select categories from {$this->db->prefix}4dealer_prices where price_id=$dealer_id");
        //return is_null($categories)?false:$categories;

        if (is_null($categories)) {
            $dealer_categories = $this->getRemoteCategories($dealer_id);
            $this->sortArrayDealerCategoriesTree($dealer_categories);

            return $dealer_categories;
        }

        $categories = unserialize($categories);
        $this->sortArrayDealerCategoriesTree($categories);

        return $categories;
    }

    public function getRemoteCategories($dealer_id)
    {
        $categories = Connector::getPriceStructure($dealer_id);
        if (sizeof($categories)) {
            $this->db->update("{$this->db->prefix}4dealer_prices", array('categories' => serialize($categories)), array('price_id' => $dealer_id));
            //echo $this->db->last_query;
            return $categories;
        }
        return false;
    }

    public function getRemotePriceList()
    {
        $get_prices = Connector::getPricesList();
        if (isset($get_prices['Result']) && empty($get_prices['Error'])) {
            foreach ($get_prices['Result'] as $price) {
                $this->updatePriceList((int)$price['id'], $price['name'], $price['last_update'], $price['till']);
            }
            return $this->getLocalPriceList();
        }
        return false;
    }

    public function updatePriceList($price_id, $name, $last_update, $subscription_end)
    {

        $update_query = $this->db->prepare("insert into {$this->db->prefix}4dealer_prices(price_id,price_name,4dealer_last_update,subscription_end) 
                                 values(%d,%s,%s,%s) 
                                 ON DUPLICATE KEY UPDATE price_name=%s, 4dealer_last_update=%s, subscription_end=%s", array($price_id, $name, $last_update, $subscription_end, $name, $last_update, $subscription_end));
        return $this->db->query($update_query);
    }

    function show_object_tree($tree, $name = null)
    {
        echo "<ul title='{$name}'>\n";
        foreach ($tree as $branch) {
            echo "<li><span id='site-term-{$branch->term_id}' data-term-id='{$branch->term_id}'>{$branch->name}</span>";
            if (isset($branch->children)) {
                $this->show_object_tree($branch->children, $branch->name);
            }
            echo "</li>\n";
        }
        echo "</ul>\n";
    }

    function show_array_tree($tree, $mappings, $local_categories)
    {
        if (!is_array($tree)) return;

        echo "<ul>\n";
        foreach ($tree as $branch) {
            $add = '';
            $class = '';
            if (isset($mappings[$branch['id']])) {
                if ($local_category = $this->search_category_by_id($local_categories, $mappings[$branch['id']])) {
                    $add = "<span>({$local_category->name})</span><span class='clear_merge' data-owner-id='{$branch['id']}'></span>";
                    $class = 'class="merged"';
                }
            }
            echo "<li class=\"".($class ? 'merged-li' : '')."\"><span data-filter-item $class id=\"dealer-category-{$branch['id']}\" data-dealer-id=\"{$branch['id']}\">{$branch['name']}</span>\n<span>$add</span>";
            if (isset($branch['children'])) {
                $this->show_array_tree($branch['children'], $mappings, $local_categories);
            }
            echo "</li>\n";
        }
        echo "</ul>\n";
    }

    function show_array_tree_select($selected_id, $tree, &$level, $options)
    {
        $level_string = '';
        for ($i = 0; $i < $level; $i++) {
            $level_string .= '.';
        }

        foreach ($tree as $branch) {
            $selected = $branch['id'] == $selected_id ? 'selected' : '';
            $options .= "<option $selected value='{$branch['id']}'>$level_string{$branch['name']}</option>\n";
            if (isset($branch['children'])) {
                $level++;
                $options = $this->show_array_tree_select($selected_id, $branch['children'], $level, $options);
            } else {
                $level = 1;
            }
        }
        return $options;
    }

    function search_category_by_id($categories, $id)
    {
        foreach ($categories as $category) {
            if ($category->term_id == $id) {
                return $category;
            }
        }
        return false;
    }

    function add_price_markup()
    {
        $result = array('res' => 0);

        $fields = [
            'dealer_id' => 'price_id',
            'category' => 'category_id',
            'price_from' => 'price_low',
            'price_to' => 'price_high',
            'price_markup' => 'markup',
            'price_is_rrc' => 'price_is_rrc',
        ];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key]) || (empty($_REQUEST[$key]) && $_REQUEST[$key] != 0)) {
                echo json_encode($result);
                wp_die();
            }

            $data[$field] = (int)$_REQUEST[$key];
        }

        $this->db->delete($this->db->prefix . "4dealer_price_markup", $data);
        if ($data['price_low'] <= $data['price_high']) {
            $result['res'] = $this->db->insert($this->db->prefix . "4dealer_price_markup", $data);
            $result['id'] = $this->db->insert_id;
        }

        echo json_encode($result);

        wp_die();
    }

    function update_price_markup()
    {
        $result = array('res' => 0);

        $fields = [
            'id' => 'id',
            'dealer_id' => 'price_id',
            'category' => 'category_id',
            'price_from' => 'price_low',
            'price_to' => 'price_high',
            'price_markup' => 'markup',
            'price_is_rrc' => 'price_is_rrc',
        ];

        foreach ($fields as $key => $field) {
            $data[$field] = (int)$_REQUEST[$key];
        }

        $result['res'] = $this->db->update("{$this->db->prefix}4dealer_price_markup", $data, ['id' => $data['id']]);

        echo json_encode($result);
        wp_die();
    }

    function delete_price_markup()
    {
        $result = array('res' => 0);

        if (!isset($_REQUEST['id'])) {
            echo json_encode($result);
            wp_die();
        }

        $result['res'] = $this->db->delete($this->db->prefix . '4dealer_price_markup', ['id' => $_REQUEST['id']]);

        echo json_encode($result);
        wp_die();
    }

    function repeat_price_markup()
    {
        $result = array('res' => 0);

        $fields = array('dealer_id' => 'price_id', 'category' => 'category_id', 'price_from' => 'price_low', 'price_to' => 'price_high', 'price_markup' => 'markup',);

        if (!isset($_REQUEST['data_string'])) {
            echo json_encode($result);
            wp_die();
        }

        $parts = explode('_', trim($_REQUEST['data_string']));
        if (sizeof($parts) != 8) {
            echo json_encode($result);
            wp_die();
        }

        $i = 3;

        foreach ($fields as $key => $value) {
            $delete_data[$value] = $parts[$i];
            $i++;
        }

        $result['res'] = $this->db->delete($this->db->prefix . "4dealer_price_markup", $delete_data);
        echo json_encode($result);
        wp_die();
    }

    function repeat_price_process()
    {
        $result = $this->update_job_by_id($_REQUEST['process_id'], ['status' => 4]);
        echo json_encode($result);
        wp_die();
    }


    function add_options_merge()
    {
        $result = array('res' => 0);

        $fields = ['dealer_id' => '4dealer_price_id', 'category' => 'section_id', 'option_name' => 'option_name', 'changed_option_name' => 'changed_option_name'];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key]) || (empty($_REQUEST[$key]) && $_REQUEST[$key] != 0)) {
                echo json_encode($result);
                wp_die();
            }

            $data[$field] = $_REQUEST[$key];
        }

        $result['res'] = $this->db->insert($this->db->prefix . '4dealer_options_merge', $data);
        $result['id'] = $this->db->insert_id;

        echo json_encode($result);

        wp_die();
    }

    /**
     * Create Ajax method.
     */
    function add_multi_options_merge_by_section_id()
    {
        $price_id = $_REQUEST['price_id'];
        $section_id = $_REQUEST['section_id'];

        $dealerCats = $this->get_dealer_categories_by_local_cat_ids($price_id, [$section_id]);

        $options = ((array) $this->get_list_options_merge($dealerCats,false));

        $optionsList = array_column($options['Result'], 'option_name');
        $optionsList = array_filter(array_unique($optionsList));

        /**
         * Получаем текущие свойства, исключаем дублирование
         */

        $existOptions = $this->get_options_merge($price_id, false, ['section_id' => $section_id]);

        $existOptionNames = array_column($existOptions, 'option_name');

        $optionsList = array_diff($optionsList, $existOptionNames);
        $optionsList = array_unique($optionsList);
        $optionsList = array_filter($optionsList);

        $data = [
            '4dealer_price_id' => $price_id,
            'section_id' => $section_id,
            'option_name' => '',
            'changed_option_name' => '',
        ];

        foreach ($optionsList as $option) {
            $data['option_name'] = $option;
            $this->db->insert($this->db->prefix . '4dealer_options_merge', $data);
        }

        echo json_encode($optionsList);

        wp_die();
    }

    function update_options_merge()
    {
        $result = array('res' => 0);

        $fields = ['id' => 'id', 'category' => 'section_id', 'option_name' => 'option_name', 'changed_option_name' => 'changed_option_name'];

        foreach ($fields as $key => $field) {
            $data[$field] = $_REQUEST[$key];
        }

        $result['res'] = $this->db->update("{$this->db->prefix}4dealer_options_merge", $data, ['id' => $data['id']]);

        echo json_encode($result);
        wp_die();
    }

    /**
     * Get options list from 4dealer
     * @param null $sectionIds
     * @param bool $ajax
     * @return array|null
     */
    function get_list_options_merge($sectionIds = null, $ajax = true)
    {
        $sectionIds = $sectionIds ? $sectionIds : [$_REQUEST['section_id']];

        if ($ajax) {
            $sectionIds = $this->get_dealer_categories_by_local_cat_ids($_REQUEST['price_id'], $sectionIds);
        }

        $result = Connector::getOptionsBySectionIdArray($sectionIds);

        if ($ajax != true)
            return $result;

        $optionsList = array_column($result['Result'], 'option_name');
        $optionsList = array_filter(array_unique($optionsList));

        echo json_encode($optionsList, JSON_UNESCAPED_UNICODE);
        wp_die();
    }


    function update_replaces_options_merge()
    {
        $result = array('res' => 1);

        $fields = array('id' => 'id', 'replaces' => 'replaces',);

        $_REQUEST['replaces'] = array_filter($_REQUEST['replaces']);

        foreach ($fields as $key => $field) {
            $data[$field] = gettype($_REQUEST[$key]) === 'array' ? json_encode($_REQUEST[$key], JSON_UNESCAPED_UNICODE) : $_REQUEST[$key];
        }

        $this->db->update("{$this->db->prefix}4dealer_options_merge", $data, ['id' => $data['id']]);

        echo json_encode($result);
        wp_die();
    }

    function delete_options_merge()
    {
        $result = array('res' => 0);

        if (!isset($_REQUEST['id'])) {
            echo json_encode($result);
            wp_die();
        }

        $result['res'] = $this->db->delete($this->db->prefix . '4dealer_options_merge', ['id' => $_REQUEST['id']]);

        echo json_encode($result);
        wp_die();
    }

    /**
     * Удаление привязок к несуществующим категориям
     */
    function delete_empty_bindings() {
        $result['res'] = $this->db->query(
            "DELETE FROM {$this->db->prefix}4dealer_categories_term WHERE term_id NOT IN (
            SELECT term_id FROM {$this->db->prefix}terms
        )");

        echo json_encode($result);
        wp_die();
    }

    function merge_categories($source_id = false, $target_id = false, $price_id = false, $ajax = true)
    {
        $result = array('res' => 1);
        $source_id = isset($_REQUEST['source_id']) ? (int)$_REQUEST['source_id'] : $source_id;
        $target_id = isset($_REQUEST['target_id']) ? (int)$_REQUEST['target_id'] : $target_id;
        $price_id = isset($_REQUEST['price_id']) ? (int)$_REQUEST['price_id'] : $price_id;

        if (isset($_REQUEST['price_id'])) {
            $categories_list = array();
            $this->buildFlatCategoriesList($this->getRemoteCategories($price_id), $categories_list);
        }


        if ($source_id && $target_id && $price_id) {
            $this->update_db_category_mapping($source_id, $target_id, $price_id);
            if (isset($categories_list)) {
                foreach ($categories_list as $category) {
                    if (in_array($source_id, $category['parents'])) {
                        $this->update_db_category_mapping($category['id'], $target_id, $price_id);
                    }
                    unset($category);
                }
            }
        }

        if ($ajax) {
            echo wp_json_encode($result);
            wp_die();
        }

        return true;
    }

    /**
     * Получение списка локальных сведенных свойств
     *
     * @param $priceId
     * @return array|false|null
     */
    function get_categories_terms_list($priceId)
    {
        $sql = "SELECT {$this->db->prefix}4dealer_categories_term.term_id, `4dealer_cat_id`, `4dealer_price_id`, name FROM {$this->db->prefix}4dealer_categories_term 
                JOIN {$this->db->prefix}terms ON {$this->db->prefix}4dealer_categories_term.term_id = {$this->db->prefix}terms.term_id
                WHERE `4dealer_price_id` = '$priceId' GROUP BY term_id";

        $res = (array) $this->db->get_results($sql);

        $termIds = array_column($res, 'term_id');
        $names = array_column($res, 'name');

        return (array) array_combine($termIds, $names);
    }

    /**
     * Получение 4dealer_cat_id из списка сведенных и переданных $categoryIds
     *
     * @param $priceId
     * @param array $categoryIds
     * @return array
     */
    function get_dealer_categories_by_local_cat_ids($priceId, $categoryIds = [])
    {
        $categoryIds = implode(',', $categoryIds);

        $sql = "SELECT {$this->db->prefix}4dealer_categories_term.term_id, `4dealer_cat_id`, `4dealer_price_id`, name FROM {$this->db->prefix}4dealer_categories_term 
                JOIN {$this->db->prefix}terms ON {$this->db->prefix}4dealer_categories_term.term_id = {$this->db->prefix}terms.term_id
                WHERE `4dealer_price_id` = '$priceId' AND {$this->db->prefix}4dealer_categories_term.term_id IN ($categoryIds)";

        $res = (array) $this->db->get_results($sql);

        return (array) array_column($res, '4dealer_cat_id');
    }

    /**
     * Получение локальной категории из списка сведенных по dealer_cat_id
     *
     * @param int $priceId
     * @param array $dealerCats
     * @return array
     */
    function get_local_cat_ids_by_dealer_cat($priceId, $dealerCats = [])
    {
        $dealerCats = implode(',', $dealerCats);

        $sql = "SELECT term_id FROM {$this->db->prefix}4dealer_categories_term 
                WHERE `4dealer_cat_id` IN ($dealerCats) AND `4dealer_price_id` = '$priceId'";

        $res = (array) $this->db->get_results($sql);

        return (array) array_column($res['rows'], 'term_id');
    }

    function update_db_category_mapping($source_id, $target_id, $price_id)
    {
        $this->db->delete($this->db->prefix . "4dealer_categories_term", array('4dealer_price_id' => $price_id, '4dealer_cat_id' => $source_id));
        $result['res'] = $this->db->insert($this->db->prefix . "4dealer_categories_term", array('term_id' => $target_id, '4dealer_price_id' => $price_id, '4dealer_cat_id' => $source_id));
        return $result;
    }

    function clear_merged_categories()
    {
        $result = array('res' => 0);
        $source_id = isset($_REQUEST['source_id']) ? (int)$_REQUEST['source_id'] : false;
        $price_id = isset($_REQUEST['price_id']) ? (int)$_REQUEST['price_id'] : false;
        if ($source_id && $price_id) {
            $result['res'] = $this->db->delete($this->db->prefix . "4dealer_categories_term", array('4dealer_price_id' => $price_id, '4dealer_cat_id' => $source_id));
        }
        echo wp_json_encode($result);
        wp_die();
    }

    function get_categories_mapping_by_price_id($price_id, $cache = false)
    {
        $result = array();

        if ($cache && isset($this->categories_mapping_by_price_cache[$price_id])) {
            return $this->categories_mapping_by_price_cache[$price_id];
        }

        $get_mappings = $this->db->get_results("select term_id,4dealer_cat_id  from {$this->db->prefix}4dealer_categories_term where 4dealer_price_id=$price_id and is_active=1");

        foreach ($get_mappings as $mapping) {
            $result[$mapping->{'4dealer_cat_id'}] = $mapping->term_id;
        }

        return $this->categories_mapping_by_price_cache[$price_id] = $result;
    }

    function add_job($cmd_list = false, $additional_data = null, $ajax = true, $parent = 0, $priority = 50)
    {

        $result = array('res' => 0);

        $priorities = array('short_price_update' => 10, 'full_price_update' => 11);

        $commands = isset($_REQUEST['cmd_list']) ? $_REQUEST['cmd_list'] : $cmd_list;

        if (isset($commands) && is_array($commands)) {
            foreach ($commands as $cmd => $data) {
                foreach ($data as $price_id) {
                    if (isset($_REQUEST['cmd_list']) && isset($priorities[$cmd])) {
                        $priority = $priorities[$cmd];
                    }
                    if ($this->db->insert($this->db->prefix . "4dealer_cmd", array('cmd' => $cmd, 'parent_id' => $parent, 'create_time' => $this->get_current_mysql_time(), 'price_id' => $price_id, 'data' => serialize($additional_data), 'priority' => $priority))) {
                        $result['data'][$cmd][$price_id] = $this->db->insert_id;
                    }
                }
            }
            if (isset($result['data'])) {
                $result['res'] = 1;
            }
        }
        if ($ajax) {
            echo wp_json_encode($result);
            wp_die();
        }
        return true;
    }

    function get_current_mysql_time()
    {
        $get_time = $this->db->get_row("select now() as now");
        return $get_time->now;
    }

    function get_jobs_queue()
    {
        $result = array();
        $get_jobs = $this->db->get_results("select cmd_id,cmd,records,status_name,price_id 
                                                      from {$this->db->prefix}4dealer_cmd, {$this->db->prefix}4dealer_statuses 
                                                      where status_id in(1,4) 
                                                      and status=status_id ");
        foreach ($get_jobs as $job) {
            $result[$job->cmd][$job->price_id] = $job;
        }

        return $result;
    }

    function get_jobs_status_by_array_id()
    {
        $result = array('res' => 0);
        if (isset($_REQUEST['cmd_list']) && is_array($_REQUEST['cmd_list'])) {
            foreach ($_REQUEST['cmd_list'] as $cmd_id) {
                $cmd_id_list[] = (int)$cmd_id;
            }
            $get_jobs = $this->db->get_results("select cmd_id,cmd,records,status_name 
                                                      from {$this->db->prefix}4dealer_cmd, {$this->db->prefix}4dealer_statuses 
                                                      where cmd_id in(" . implode(',', $cmd_id_list) . ") 
                                                      and status=status_id");
            foreach ($get_jobs as $job) {
                $result['data'][$job->cmd_id] = $job;
                if ($job->cmd == 'full_price_update' || $job->cmd == 'short_price_update') {
                    $total_records = $this->db->get_var("select sum(records)  
                                                      from {$this->db->prefix}4dealer_cmd  
                                                      where parent_id = {$job->cmd_id} and cmd!='get_products_info'");
                    $count_wait_and_run_jobs = $this->db->get_var("select count(cmd_id)  
                                                      from {$this->db->prefix}4dealer_cmd  
                                                      where parent_id = {$job->cmd_id} and status in (1,4) and cmd!='get_products_info'");
                    $result['data'][$job->cmd_id]->records = is_null($total_records) ? 0 : $total_records;

                    if ($count_wait_and_run_jobs == 0 && !($job->status_name == 'Running' || $job->status_name == 'Waiting')) {
                        $result['data'][$job->cmd_id]->status_name = 'Finished';
                    } else {
                        //var_dump($job);
                        if ($job->status_name == 'Finished') {
                            $result['data'][$job->cmd_id]->status_name = 'Running';
                        }
                    }
                }
            }
            if (isset($result['data'])) {
                $result['res'] = 1;
            }
        }
        echo wp_json_encode($result);
        wp_die();
    }

    function dealer_categories_tree_to_term($tree, $price_id, $parent = 0, $dealer_section = 0, $dealer_section_child=false)
    {
        //var_dump($tree);
//        var_dump($parent);
//        var_dump($dealer_section);
//        var_dump($dealer_section_child);
//        flush();
        foreach ($tree as $branch) {
            $name = preg_replace('/\s+/u', ' ', trim($branch['name']));
            $cat_tax = $this->rus2translit($name) . "_{$branch['id']}";
            //$cat_tax=$this->rus2translit($name);
            $new_cat = term_exists($name, 'product_cat', $parent);
            if ($dealer_section == 0 || $dealer_section == $branch['id'] || $dealer_section_child == true) {
                if (!is_array($new_cat)) {
                    $new_cat = wp_insert_term($name, 'product_cat', array('slug' => $cat_tax . "_$parent", 'parent' => $parent));
                    if (!is_wp_error($new_cat)) {
                        $this->merge_categories($branch['id'], $new_cat['term_id'], $price_id, false);
                    }
                } elseif (!is_wp_error($new_cat)) {
                    $this->merge_categories($branch['id'], $new_cat['term_id'], $price_id, false);
                }
            }
            
            
            if (isset($branch['children'])) {
                if (!is_wp_error($new_cat)) {
                    $this->dealer_categories_tree_to_term($branch['children'], $price_id, $new_cat['term_id'], $dealer_section ,($dealer_section == $branch['id'] || $dealer_section_child == true));
                } else {
                    var_dump($new_cat);
                    die();
                }
            }
            clean_term_cache( $name, 'product_cat' );
            if ($dealer_section == $branch['id']) {
                break;
            }
        }

        return true;
    }

    function import_dealer_categories_tree()
    {
        @ini_set('max_execution_time','0');
        $result = array('res' => 0);
        if (isset($_REQUEST['dealer_id'])) {
            $res = $this->dealer_categories_tree_to_term(
                    $this->getLocalDealerCategories((int)$_REQUEST['dealer_id']), 
                    (int)$_REQUEST['dealer_id'],
                    0,
                    isset($_REQUEST['section_id'])?intval($_REQUEST['section_id']):0
            );
            if ($res) {
                $result['res'] = 1;
            }
        }
        echo wp_json_encode($result);
        wp_die();
    }

    function rus2translit($string)
    {
        $converter = array('а' => 'a', 'б' => 'b', 'в' => 'v', 'г' => 'g', 'д' => 'd', 'е' => 'e', 'ё' => 'e', 'ж' => 'zh', 'з' => 'z', 'и' => 'i', 'й' => 'y', 'к' => 'k', 'л' => 'l', 'м' => 'm', 'н' => 'n', 'о' => 'o', 'п' => 'p', 'р' => 'r', 'с' => 's', 'т' => 't', 'у' => 'u', 'ф' => 'f', 'х' => 'h', 'ц' => 'c', 'ч' => 'ch', 'ш' => 'sh', 'щ' => 'sch', 'ь' => '\'', 'ы' => 'y', 'ъ' => '\'', 'э' => 'e', 'ю' => 'yu', 'я' => 'ya', 'А' => 'A', 'Б' => 'B', 'В' => 'V', 'Г' => 'G', 'Д' => 'D', 'Е' => 'E', 'Ё' => 'E', 'Ж' => 'Zh', 'З' => 'Z', 'И' => 'I', 'Й' => 'Y', 'К' => 'K', 'Л' => 'L', 'М' => 'M', 'Н' => 'N', 'О' => 'O', 'П' => 'P', 'Р' => 'R', 'С' => 'S', 'Т' => 'T', 'У' => 'U', 'Ф' => 'F', 'Х' => 'H', 'Ц' => 'C', 'Ч' => 'Ch', 'Ш' => 'Sh', 'Щ' => 'Sch', 'Ь' => '\'', 'Ы' => 'Y', 'Ъ' => '\'', 'Э' => 'E', 'Ю' => 'Yu', 'Я' => 'Ya',);
        return strtr($string, $converter);
    }

    function job_checker()
    {
        gc_enable();
        ini_set('soap.wsdl_cache_enabled', '0');
        ini_set('soap.wsdl_cache_ttl', '0');

        $this->cancel_dead_commands();
        $get_scheduled_prices = $this->db->get_results("SELECT price_id FROM {$this->db->prefix}4dealer_prices where schedule_update_time=HOUR(NOW()) and local_last_update<4dealer_last_update");
        if (!empty($get_scheduled_prices)) {
            $prices = array();
            foreach ($get_scheduled_prices as $price) {
                $prices[] = $price->price_id;
            }
            $ids = implode(',', $prices);
            $check_for_scheduled_running_tasks = $this->db->get_results("SELECT price_id FROM {$this->db->prefix}4dealer_cmd where price_id in($ids) and DATE(create_time)=DATE(NOW()) 
                                                                              and HOUR(Now())=Hour(create_time) and cmd='full_price_update' and status in(1,2,3,4)");
            $runned = array();

            if (!empty($check_for_scheduled_running_tasks)) {
                foreach ($check_for_scheduled_running_tasks as $price) {
                    $runned[] = $price->price_id;
                }
            }
            $not_runned_ids = array_diff($prices, $runned);
            if (sizeof($not_runned_ids)) {
                //$this->add_job(array('short_price_update' => $not_runned_ids), null, false, 0, 25);
                $this->add_job(array('full_price_update' => $not_runned_ids), array('data_filter' => true), false, 0, 26);
            }
        }
        $count_running_jobs = $this->db->get_var("select count(*) from {$this->db->prefix}4dealer_cmd where status=1");
        if (is_null($count_running_jobs) || $count_running_jobs <= $this->MAX_PROCESSES_PER_JOBS) {
            $get_queue = $this->db->get_results("select * from {$this->db->prefix}4dealer_cmd where status=4 order by priority, create_time limit " . $this->MAX_PROCESSES_PER_JOBS);
            $jobs_for_run = array();
            $flag = false;
            if (!wp_defer_term_counting()) {
                wp_defer_term_counting(true);
                wp_defer_comment_counting(true);
                $flag = true;
            }
            foreach ($get_queue as $job) {
                if (method_exists($this, $job->cmd)) {

                    // get_products_info можно запускать только 1
                    if (in_array('get_products_info', array_column($jobs_for_run, 'cmd'))) {
                        continue;
                    }

                    $this->logger("[$job->cmd_id] {$job->cmd} found");
                    $res = $this->update_job_by_id($job->cmd_id, array('status' => 1, 'pid' => $this->getPid($job->cmd_id), 'start_time' => $this->get_current_mysql_time()));
                    $jobs_for_run[] = $job;
                }
            }
            foreach ($jobs_for_run as $job) {
                $this->update_job_by_id($job->cmd_id, array('status' => 1, 'pid' => $this->getPid($job->cmd_id), 'start_time' => $this->get_current_mysql_time()));
                $this->logger("[$job->cmd_id] update: status => 1, pid => " . $this->getPid($job->cmd_id) . ", start_time => {$this->get_current_mysql_time()}, res => " . print_r($res, true));
                $job->pid = $this->getPid($job->cmd_id);
                $pidChk = $this->check_pid($job->pid);
                $this->logger("[$job->cmd_id] pidChk: " . print_r($pidChk, true));
                $this->{$job->cmd}($job);
                $this->logger("[$job->cmd_id] done!");
            }
            if ($flag) {
                wp_defer_term_counting(false);
                wp_defer_comment_counting(false);
            }
            exit();
        }
        exit();
    }

    function getPid($cmd_id)
    {
        if (get_option("{$this->plugin_name}-4dealer_pid_method", 1) == 1) {
            return getmypid();
        } else {
            if (!is_dir(WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/')) {
                mkdir(WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/');
            }
            $pidFile = WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/' . $cmd_id;
            if (!is_file($pidFile)) {
                file_put_contents(WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/' . $cmd_id, '1');
                if (!isset($_SESSION['4D_PID']) || !in_array($cmd_id, $_SESSION['4D_PID'])) {
                    $_SESSION['4D_PID'][] = $cmd_id;
                }
            }
            return $cmd_id;
        }
    }

    function update_available_distributors()
    {
        $this->clear_cmd_table();
        $get_prices = Connector::getPricesList();
        if (isset($get_prices['Result']) && empty($get_prices['Error'])) {
            foreach ($get_prices['Result'] as $price) {
                $this->updatePriceList((int)$price['id'], $price['name'], $price['last_update'], $price['till']);
            }
            exit();
        }
        exit();
    }

    function get_job_status_by_id($id)
    {
        return $this->db->get_var("select status from {$this->db->prefix}4dealer_cmd where cmd_id=$id");
    }

    function update_job_by_id($job_id, $data)
    {
        return $this->db->update("{$this->db->prefix}4dealer_cmd", $data, array('cmd_id' => $job_id));
    }

    function update_jobs_by_parent_id($parent_id, $data)
    {
        return $this->db->update("{$this->db->prefix}4dealer_cmd", $data, array('parent_id' => $parent_id));
    }

    function cancel_dead_commands()
    {

        $get_jobs = $this->db->get_results("select cmd_id,pid 
                                                      from {$this->db->prefix}4dealer_cmd, {$this->db->prefix}4dealer_statuses 
                                                      where status_id=1 
                                                      and status=status_id ");
        foreach ($get_jobs as $job) {
            if (!$this->check_pid($job->pid)) {
                $this->logger("[$job->cmd_id] Завершаем зависший процесс!");
                $this->cancel_command($job->cmd_id, false);
            }
        }
    }

    function check_pid($pid)
    {
        if ($this->defaultPidMethod == true) {
            return $this->check_pid_def($pid);
        } else {
            return $this->check_pid_adv($pid);
        }
    }

    function check_pid_def($pid)
    {
        $file_path = "/proc/$pid/cmdline";
        if (is_file($file_path)) {
            $res = file_get_contents($file_path);
            if ($res) {
                return strpos($res, 'wp-cron.php');
            } else {
                return true;
            }
        } else {
            $this->logger("Pid file not found!");
        }
        return false;
    }

    function check_pid_adv($pid)
    {
        return is_file(WP_CONTENT_DIR . '/plugins/wp-fdeal/pid/' . $pid);
    }

    function process_killer($job_id, $current_records)
    {
        $current_status = $this->get_job_status_by_id($job_id);
        if ($current_status == 5) {
            $data = array('records' => $current_records, 'end_time' => $this->get_current_mysql_time());
            $this->update_job_by_id($job_id, $data);
            exit();
        }
    }

    function cancel_command($cmd_id = false, $ajax = true)
    {
        $result['res'] = 0;
        $id = isset($_REQUEST['cmd_id']) ? (int)$_REQUEST['cmd_id'] : $cmd_id;
        if ($id) {
            $data = array('status' => 5);
            //if ($this->update_job_by_id($id, $data) && $this->update_jobs_by_parent_id($id, $data)) {
            if ($this->update_job_by_id($id, $data)) {
                $result['res'] = 1;
            }
        }
        if ($ajax) {
            echo wp_json_encode($result);
            wp_die();
        }
    }

    function get_dealer_ids_by_price_id($price_id)
    {
        $get_ids = $this->db->get_results("select post_id,meta_value from {$this->db->prefix}postmeta  where meta_key='_sku' and post_id in (SELECT distinct post_id FROM {$this->db->prefix}postmeta where meta_key='_price_id' and meta_value=$price_id)");
        if (!is_null($get_ids) && !empty($get_ids)) {
            foreach ($get_ids as $dealer_id) {
                $ids[$dealer_id->meta_value] = $dealer_id->post_id;
            }
            return $ids;
        }
        return false;
    }

    function clear_stock_for_unupdated_products($price_id, $date)
    {
        $ids = $this->get_dealer_ids_by_price_id($price_id);
        if (!$ids) {
            return false;
        }

        $post_ids = implode(',', array_values($ids));

        $get_unupdated_ids = $this->db->get_results("select ID from {$this->db->prefix}posts where ID in ($post_ids) and post_modified<'$date'");
        if (!$get_unupdated_ids) {
            return false;
        }
        $ids = array();
        if (is_null($get_unupdated_ids)) {
            return false;
        }
        foreach ($get_unupdated_ids as $post_id) {
            $ids[] = $post_id->ID;
        }

        $post_ids = implode(',', $ids);

        $this->db->query("update {$this->db->prefix}postmeta set meta_value=0
                                where meta_key='_stock' and post_id in($post_ids)");
        $this->db->query("update {$this->db->prefix}postmeta set meta_value='outofstock' 
                                where meta_key='_stock_status' and post_id in($post_ids)");
    }

    function update_post_date($post_id)
    {
        $time = $this->get_current_mysql_time();
        $gmt_time = get_gmt_from_date($time);
        $this->db->query("UPDATE {$this->db->prefix}posts set post_modified='$time',post_modified_gmt='$gmt_time' where id=$post_id");
    }

    function create_new_attribute($attribute)
    {
        // Check_admin_referer('woocommerce-add-new_attribute');
        if (empty($attribute['attribute_type'])) {
            $attribute['attribute_type'] = 'text';
        }

        if (empty($attribute['attribute_orderby'])) {
            $attribute['attribute_orderby'] = 'menu_order';
        }

        if (empty($attribute['attribute_public'])) {
            $attribute['attribute_public'] = isset($attribute['attribute_public']) ? intval($attribute['attribute_public']) : 0;
        }

        if (empty($attribute['attribute_name']) || empty($attribute['attribute_label'])) {
            $this->logger("-- Error! Please, provide an attribute name and slug.");
            return new WP_Error('error', __('Please, provide an attribute name and slug.', 'woocommerce'));
        } elseif (($valid_attribute_name = $this->valid_attribute_name($attribute['attribute_name'])) && is_wp_error($valid_attribute_name)) {
            return $valid_attribute_name;
        } elseif (taxonomy_exists(wc_attribute_taxonomy_name($attribute['attribute_name']))) {
            $attr_id = wc_attribute_taxonomy_id_by_name(wc_attribute_taxonomy_name($attribute['attribute_name']));
            $this->logger("-- wc_attribute_taxonomy_name exist: ". $attr_id.'. Do update!');
            $this->db->update($this->db->prefix . 'woocommerce_attribute_taxonomies', $attribute, ['attribute_id'=>$attr_id]);
            do_action('woocommerce_attribute_updated', $attr_id, $attribute, $attribute['attribute_name']);
        } else {
            $this->db->insert($this->db->prefix . 'woocommerce_attribute_taxonomies', $attribute);
            do_action('woocommerce_attribute_added', $this->db->insert_id, $attribute);
        }
        
        flush_rewrite_rules();
        delete_transient('wc_attribute_taxonomies');

        return true;
    }

    function valid_attribute_name($attribute_name)
    {
        if (strlen($attribute_name) >= 28) {
            return new WP_Error('error', sprintf(__('Slug "%s" is too long (28 characters max). Shorten it, please.', 'woocommerce'), sanitize_title($attribute_name)));
        } elseif (wc_check_if_attribute_name_is_reserved($attribute_name)) {
            return new WP_Error('error', sprintf(__('Slug "%s" is not allowed because it is a reserved term. Change it, please.', 'woocommerce'), sanitize_title($attribute_name)));
        }

        return true;
    }

    function create_needed_taxonomies()
    {
        $taxonomies = array(
            'pa_part_no' => array('attribute_name' => 'part_no', 'attribute_label' => 'Артикул', 'attribute_type' => 'text', 'attribute_orderby' => 'menu_order', 'attribute_public' => false),
            'pa_no' => array('attribute_name' => 'no', 'attribute_label' => 'Код поставщика', 'attribute_type' => 'text', 'attribute_orderby' => 'menu_order', 'attribute_public' => false),
            'pa_brand' => array('attribute_name' => 'brand', 'attribute_label' => 'Бренд', 'attribute_type' => 'select', 'attribute_orderby' => 'menu_order', 'attribute_public' => false),);

        foreach ($taxonomies as $key => $value) {
            if (!taxonomy_exists($key)) {
                $this->create_new_attribute($value);
            }
        }
    }

    /**
     * Добавление аттрибутов для товара (включает проверку и логику работы "Сведения свойств")
     * @param $product
     * @param $price_id
     * @param $product_id
     * @param $attributes (name, value)
     * @param $product_local_cat
     * @param $data
     */
    function add_product_taxonomies($product, $price_id, $product_id, $attributes, $product_local_cat, $data)
    {

        global $wp_taxonomies;

        $wc_attributes = [];
        $priceInfo = $this->getPriceInfoByCache($price_id);
        
        foreach ($attributes as $attribute) {
            $this->taxonomies_merge_replaces($price_id, $attribute, $product_local_cat);
            //$this->logger("[$data->cmd_id] {$attribute->name}.");
            $translitName = mb_strtolower($this->rus2translit(trim($attribute->name)));
            $attribute_name = wc_attribute_taxonomy_name($translitName);
            $attribute_name = mb_substr($attribute_name, 0, 24); //максимальная длина <= 27 ('pa_...')
            $attribute_name = rtrim(trim($attribute_name), '-');
            $taxonomy = 'pa_'.$attribute_name;
            
            $createAttribute = [
                'attribute_name' => $attribute_name,
                'attribute_label' => $attribute->name,
                'attribute_type' => 'text',
                'attribute_orderby' => 'menu_order',
                'attribute_public' => intval($priceInfo['public_options']),
            ];

            if (!taxonomy_exists($attribute_name)) {
                $this->logger("[$data->cmd_id] Добавляем новый параметр {$attribute_name}[{$taxonomy}].");
                $res = $this->create_new_attribute($createAttribute);

                if ($res) {
                    $wp_taxonomies[$attribute_name] = true;
                }
            } else {
                $this->logger("[$data->cmd_id] Параметр {$attribute_name}[{$taxonomy}] найден.");
            }

            wp_set_object_terms($product_id, $attribute->value, $taxonomy, true);
            $this->addProductAttributeToArray($wc_attributes, $taxonomy, $attribute->value);
        }

        if (sizeof($wc_attributes)) {
            $product->set_attributes($wc_attributes);
            $product->save();
        }

    }

    function addProductAttributeToArray(&$attributes, $name, $value)
    {
        $attribute_object = new WC_Product_Attribute();

        $attribute_object->set_name($name);
        $attribute_object->set_options(explode(WC_DELIMITER, $value));
        $attribute_object->set_position(count($attributes) + 1);
        $attribute_object->set_visible(1);
        $attribute_object->set_variation(0);

        $attributes[] = $attribute_object;
    }

    function taxonomies_merge_replaces($price_id, &$attribute, $product_local_cat)
    {
        $options_merge = $this->get_options_merge($price_id, true);

        foreach ($options_merge as $option) {

            if ($option['section_id'] != $product_local_cat && $option['section_id'] != '0') {
                continue;
            }

            if ($option['option_name'] == $attribute->name || $option['changed_option_name'] == $attribute->name) {

                if ($option['option_name'] == $attribute->name && !empty($option['changed_option_name'])) {
                    $attribute->name = $option['changed_option_name'];
                }

                if (!empty($option['replaces'])) {
                    $option['replaces'] = json_decode($option['replaces'], true);

                    foreach ($option['replaces'] as $find => $replace) {
                        $attribute->value = str_replace($find, $replace, $attribute->value);
                    }
                }

            }

        }

    }

    function short_price_update($data)
    {
        ini_set('max_execution_time', 300);
        $num_rows = get_option("{$this->plugin_name}-4dealer_short_step", 500);
        $product_ids = $this->get_dealer_ids_by_price_id($data->price_id);
        if (!$product_ids) {
            $this->full_price_update($data);
            return true;
        }
        $this->create_needed_taxonomies();

        $price_markups = $this->get_categorized_price_markups($data->price_id);
        $price_info = $this->get_price_info_by_id($data->price_id);

        $rows_count = sizeof($product_ids);

        $ids = array_keys($product_ids);

        $iteration_count = ceil($rows_count / $num_rows);
        $k = 0;

        $last_step = false;

        for ($i = 0; $i < $iteration_count; $i++) {

            $target_ids = array();
            for ($j = $i * $num_rows; $j < ($i + 1) * $num_rows && $j < $rows_count; $j++) {
                $target_ids[] = $ids[$j];
                $k++;
            }
            if ($k == $rows_count) {
                $last_step = true;
            }
            $this->add_job(array('update_products' => array($data->price_id)), array($target_ids, $price_markups, $price_info, $last_step), false, $data->cmd_id);
        }

        //$this->add_job(array('clear_old_products' => array($data->price_id)), array($data->price_id, $data->create_time), false, $data->cmd_id, 54);

        return $this->set_command_result($data->cmd_id, $k, $data->price_id, false, 'short_price_update');
    }

    function clear_old_products($data)
    {
        $target_data = unserialize($data->data);
        $this->clear_stock_for_unupdated_products($target_data[0], $target_data[1]);
        return $this->set_command_result($data->cmd_id, 0, $data->price_id, false, 'clear_old_products');
    }

    function clear_cmd_table()
    {
        $this->db->query("DELETE FROM {$this->db->prefix}4dealer_cmd where end_time < now() - interval 24 hour and status not in(1,4)");
    }

    function update_products($data)
    {
        $target_data = unserialize($data->data);
        $product_ids = $this->get_dealer_ids_by_price_id($data->price_id);
        $products_controller = new WC_REST_Products_Controller();
        $wp_put_rest_request = new WP_REST_Request('PUT');
        $k = 0;

        $fetch_products = connector::getPriceItemsByIdArray($target_data[0]);
        unset($target_data[0]);

        if (isset($fetch_products['Result']) && empty($fetch_products['Error'])) {
            //$this->db->query('SET autocommit = 0;');

            foreach ($fetch_products['Result'] as $product_data) {

                $general_data = [
                    'small_quantity_transform' => $target_data[2]['small_quantity_transform'],
                    'medium_quantity_transform' => $target_data[2]['medium_quantity_transform'],
                    'large_quantity_transform' => $target_data[2]['large_quantity_transform'],
                    'price_name' => $target_data[2]['price_name'], 'post_id' => $product_ids[$product_data['id']]
                ];

                $product_data = array_merge($product_data, $general_data);

                $product = new dealerProduct($product_data, $this->plugin_name, $this->getPriceInfoByCache($product_data['price_id']));
                $product->preWork();
                $product->recalculatePrice($target_data[2]['price_markup'], $target_data[1]);

                $product->roundPrice();

                if ($product->update(false, $products_controller, $wp_put_rest_request, true, $this->db)) {
                    $k++;
                    $this->update_post_date($product_data['post_id']);
                }
                $product_data = null;
                $product = null;
                $general_data = null;
            }

            $fetch_products = null;
            $product_ids = null;
            //$this->db->query('COMMIT;');
            //$this->db->query('SET autocommit = 1;');
            $this->process_killer($data->cmd_id, $k);
            $this->update_job_by_id($data->cmd_id, array('records' => $k));
        }

        gc_collect_cycles();

        if ($target_data[3]) {
            $this->add_job(array('clear_old_products' => array($data->price_id)), array($data->price_id, $data->create_time), false, $data->cmd_id, 54);
            $count_running_jobs = $this->db->get_var("select count(*) from {$this->db->prefix}4dealer_cmd where status in (1,4) and cmd='hide_duplicates'");
            if ((is_null($count_running_jobs) || $count_running_jobs == 0) && sizeof($this->getLocalPriceList()) > 1) {
                $this->add_job(array('hide_duplicates' => array($data->price_id)), null, false, 0, 55);
            }
        }

        return $this->set_command_result($data->cmd_id, $k, $data->price_id, false, 'update_products');
    }

    function convert($size)
    {
        $unit = array('b', 'kb', 'mb', 'gb', 'tb', 'pb');
        return @round($size / pow(1024, ($i = floor(log($size, 1024)))), 2) . ' ' . $unit[$i];
    }

    function set_command_result($cmd_id, $records, $price_id, $date_filter = false, $cmd = '')
    {
        $timestamp = $this->get_current_mysql_time();
        $key = $cmd == 'short_price_update' ? 'local_last_update' : 'local_last_full_update';
        $this->update_job_by_id($cmd_id, array('status' => 2, 'records' => $records, 'end_time' => $timestamp));
        $this->db->update("{$this->db->prefix}4dealer_prices", array($key => $timestamp), array('price_id' => $price_id));
    }

    function update_schedule_date()
    {
        $result['res'] = 0;
        $price_id = isset($_REQUEST['dealer_id']) ? (int)$_REQUEST['dealer_id'] : false;
        $update_hour = isset($_REQUEST['update_hour']) ? (int)$_REQUEST['update_hour'] : -1;
        if ($price_id && $update_hour < 25 && $update_hour > -2) {
            $this->db->update("{$this->db->prefix}4dealer_prices", array('schedule_update_time' => $update_hour), array('price_id' => $price_id));
            $result['res'] = 1;
        }

        echo wp_json_encode($result);
        wp_die();
    }

    function update_dealer_params()
    {
        $result['res'] = 0;
        $price_id = isset($_REQUEST['dealer_id']) ? (int)$_REQUEST['dealer_id'] : false;

        if (!$price_id) {
            echo wp_json_encode($result);
            wp_die();
        }

        $data = [];

        $mapper = [
            'small_quantity' => 'small_quantity_transform',
            'medium_quantity' => 'medium_quantity_transform',
            'large_quantity' => 'large_quantity_transform',
            'price_markup' => 'price_markup',
            'round_price' => 'round_price',
            'round_rule' => 'round_rule',
            'product_add_table' => 'product_add_table',
            'auto_load_options' => 'auto_load_options',
            'public_options' => 'public_options',
        ];

        foreach ($mapper as $key => $value) {
            if (isset($_REQUEST[$key])) {
                $data[$value] = (int)$_REQUEST[$key];
            }
        }

        if (!sizeof($data)) {
            echo wp_json_encode($result);
            wp_die();
        }

        $this->db->update("{$this->db->prefix}4dealer_prices", $data, ['price_id' => $price_id]);
        $result['res'] = 1;

        echo wp_json_encode($result);
        wp_die();
    }

    function setInactiveCategories($id_list, $price_id)
    {
        $this->db->query("update {$this->db->prefix}4dealer_categories_term set is_active=1 where 4dealer_price_id=$price_id");

        if (sizeof($id_list)) {
            $this->db->query("update {$this->db->prefix}4dealer_categories_term set is_active=0 where 4dealer_price_id=$price_id and 4dealer_cat_id not in(" . implode(',', $id_list) . ")");
        }
    }

    function full_price_update($data, $use_date_filter = false)
    {
        @ini_set('memory_limit', '750M');
        @ini_set('max_execution_time', 1800);

        $this->logger("[$data->cmd_id] Полное обновление прайса запущено.");

        $use_date_filter = false;
        $num_rows = get_option("{$this->plugin_name}-4dealer_full_step", 100);;
        $i = 1;
        $limit = $num_rows;
        $error = false;

        $dealer_categories = array();

        $this->create_needed_taxonomies();
        $this->logger("[$data->cmd_id] Создаем необходимые таксономии...");

        $this->buildFlatCategoriesList($this->getRemoteCategories($data->price_id), $dealer_categories);
        $this->logger("[$data->cmd_id] Получаем список категорий...");

        $this->setInactiveCategories(array_keys($dealer_categories), $data->price_id);

        $mappings = $this->get_categories_mapping_by_price_id($data->price_id);
        $this->logger("[$data->cmd_id] Получаем связи категорий...");

        $mapping_keys = array_keys($mappings);
        $price_info = $this->get_price_info_by_id($data->price_id);
        $this->logger("[$data->cmd_id] Получаем информацию о каталоге...");

        if (!is_array($data->data)) {
            $data->data = unserialize($data->data);
        }
        //$date_filter = $use_date_filter || (isset($data->data['data_filter']) && $data->data['data_filter']) ? $price_info['local_last_full_update'] : false;
        $date_filter = false;
        if ($date_filter) {
            $this->add_job(array('short_price_update' => array($data->price_id)), null, false, 0, 25);
            $this->logger("[$data->cmd_id] Дополнительно создаем задачу быстрого обновления...");
        }
        $general_data = array('small_quantity_transform' => $price_info['small_quantity_transform'], 'medium_quantity_transform' => $price_info['medium_quantity_transform'], 'large_quantity_transform' => $price_info['large_quantity_transform'], 'price_name' => $price_info['price_name']);

        $not_last_step = true;

        if (empty($mapping_keys)) {
            $this->logger("[$data->cmd_id] Нет ни одной сопоставленной категории для данного дистрибьютора. Пожалуйста выполните сопостоваление.");
            $this->update_job_by_id($data->cmd_id, array('status' => 3, 'end_time' => $this->get_current_mysql_time()));
            return false;
        }

        $totalRows = 0;


        do {
            $this->logger("[$data->cmd_id] Цикл получения записей товаров {$i}:");
            $array_data = Connector::getPriceItems($data->price_id, $date_filter, false, $i, $mapping_keys, $limit);
            $this->logger("[$data->cmd_id] Запрашиваем список записей...");
            if ($array_data['Info']['numRows'] != $limit) {
                $this->logger("[$data->cmd_id] Повторная попытка запроса записей...");
                sleep(1);
                $array_data = Connector::getPriceItems($data->price_id, $date_filter, false, $i, $mapping_keys, $limit);
                $this->logger("[$data->cmd_id] Повторная попытка успешна.");
            }

            $stepRowsNum = count($array_data['Result']);
            $totalRows += $stepRowsNum;

            $this->logger("[$data->cmd_id] Количество записей: " . $stepRowsNum);

            if (isset($array_data['Result']) && empty($array_data['Error'])) {
                if ($array_data['Info']['numRows'] != $limit) {
                    $not_last_step = false;
                    $this->logger("[$data->cmd_id] Отмечаем шаг последним.");
                }

                $this->add_job(array('create_products' => array($data->price_id)), array($array_data, $general_data, $price_info, $mappings, $dealer_categories, !$not_last_step, $date_filter), false, $data->cmd_id);
                $this->logger("[$data->cmd_id] Добавляем задачу create_products...");

                $this->update_job_by_id($data->cmd_id, array('records' => $stepRowsNum));
                $this->logger("[$data->cmd_id] Устанавливаем количество обработанных записей:" . $stepRowsNum );
                $i++;
            } else {
                $error = true;
                $this->logger("[$data->cmd_id] Ошибка получения данных.");
                $this->logger(print_r($array_data['Result'], true));
            }
            $this->logger("[$data->cmd_id] Шаг окончен.");
        } while ($error == false && $not_last_step);
        if (!$error) {
            $this->logger("[$data->cmd_id] Всего обработано:" . $stepRowsNum );
            $this->set_command_result($data->cmd_id, $stepRowsNum, $data->price_id, $date_filter, 'full_price_update');
            $this->logger("[$data->cmd_id] Обработка окончена!");
            return true;
        }
        $this->update_job_by_id($data->cmd_id, array('status' => 3, 'data' => serialize($array_data['Error']), 'records' => $stepRowsNum, 'end_time' => $this->get_current_mysql_time()));


        $this->logger("Ошибка задачи [$data->cmd_id]: ");
        $this->logger($array_data['Error']);
        $this->logger("In SOAP: ");
        $this->logger(array($data->price_id, $date_filter, false, $i, $mapping_keys, $limit));
        return false;
    }

    function create_products($data)
    {
        $this->logger("[$data->cmd_id] Запущена задача обработки товаров!");
        $products_controller = new WC_REST_Products_Controller();
        $wp_rest_request = new WP_REST_Request('POST');
        $wp_put_rest_request = new WP_REST_Request('PUT');
        //$variations_controler = class_exists('WC_REST_Product_Variations_Controller') ? new WC_REST_Product_Variations_Controller():false;
        $variations_controler = new WC_REST_Product_Variations_Controller();

        $exclude_str = get_option("{$this->plugin_name}-4dealer_exclude_str", true);
        $exclude_str = strlen($exclude_str) > 0 ? explode(',', $exclude_str) : array();


        $target_data = unserialize($data->data);
        $price_markups = $this->get_categorized_price_markups($data->price_id);
        $new_products = array();
        $j = 0;

        if (isset($target_data[0]['Result']) && empty($target_data[0]['Error'])) {
            $this->logger("[$data->cmd_id] Товаров для обработки:" . count($target_data[0]['Result']));
            foreach ($target_data[0]['Result'] as $key => $product_data) {
                $this->logger("[$data->cmd_id] Обрабатываем товар:" . $product_data['name']);
                /*if ($product_data['my_prices']['price_rub'] <= 1000) {
                    $this->logger("[$data->cmd_id] Пропускаем товар с ценой меньше 1000 руб.");
                    continue;
                }*/
                $product_data = array_merge($product_data, $target_data[1]);
                $product_id = wc_get_product_id_by_sku($product_data['id']);
                if ($product_id) {
                    $this->logger("[$data->cmd_id] Товар найден в базе ($product_id). Обновляем.");
                    $product_data['post_id'] = $product_id;
                    $this->update_post_date($product_data['post_id']);
                }

                $product = new dealerProduct($product_data,$this->plugin_name, $this->getPriceInfoByCache($product_data['price_id']));

                $product->recalculatePrice($target_data[2]['price_markup'], $price_markups);

                $product->roundPrice();

                if ($this->exclude_str_chk($product_data, $exclude_str, $data->cmd_id)) {
                    if ($resSave = $product->save($target_data[3], $target_data[4], $products_controller, $wp_rest_request, $wp_put_rest_request, $variations_controler, $this->db)) {
                        $this->logger("[$data->cmd_id] Товар добавлен.");
                        $new_products[] = $product_data['id'];
                        $j++;
                    } else {
                        $this->logger("[$data->cmd_id] " . var_export($resSave, true));
                        //$this->logger("[$data->cmd_id] ".print_r($products_controller,true));
                        $this->logger("[$data->cmd_id] Ошибка сохранения товара.");
                    }
                } else {
                    $this->logger("[$data->cmd_id] Пропускем по фильтру названия.");
                }
                unset($product_data, $product_id, $product, $target_data['Result'][$key]);
            }
            unset($array_data);


            $this->process_killer($data->cmd_id, $j);
            if (sizeof($new_products)) {
                $this->logger("[$data->cmd_id] Создаем задачу создания карточек новых товаров.");
                $this->add_job(array('get_products_info' => array($data->price_id)), $new_products, false, $data->cmd_id);
            }

            $this->update_job_by_id($data->cmd_id, array('records' => $j));
            if ($target_data[5] && !$target_data[6]) {
                $this->logger("[$data->cmd_id] Создаем задачу отключения отсутствующих товаров.");
                $this->add_job(array('clear_old_products' => array($data->price_id)), array($data->price_id, $data->create_time), false, $data->cmd_id, 54);
                $count_running_jobs = $this->db->get_var("select count(*) from {$this->db->prefix}4dealer_cmd where status in (1,4) and cmd='hide_duplicates'");
                if ((is_null($count_running_jobs) || $count_running_jobs == 0) && sizeof($this->getLocalPriceList()) > 1) {
                    $this->add_job(array('hide_duplicates' => array($data->price_id)), null, false, 0, 55);
                    $this->logger("[$data->cmd_id] Создаем задачу поиска дубликатов.");
                }
            }
            $this->db->flush();
            return $this->set_command_result($data->cmd_id, $j, $data->price_id, false, 'create_products');
        }
        $this->update_job_by_id($data->cmd_id, array('status' => 3, 'data' => serialize($target_data[0]['Error']), 'records' => $j, 'end_time' => $this->get_current_mysql_time()));

        return false;
    }

    function exclude_str_chk($product_data, $exclude_str, $cmd_id)
    {
        if (!empty($exclude_str)) {
            foreach ($exclude_str as $excl) {
                if (strpos(strtolower($product_data['name']), strtolower(trim($excl))) !== false) {
                    $this->logger("[$cmd_id] Найдено стоп-слово '{$excl}'.");
                    return false;
                }
            }
        }
        return true;
    }

    function get_price_info_by_id($id)
    {
        $price = $this->db->get_row("select * from {$this->db->prefix}4dealer_prices where price_id=" . (int)$id, ARRAY_A);
        return is_null($price) ? false : $price;
    }

    function get_options_merge($dealer_id, $cache = false, $options = [])
    {
        if ($cache && isset($this->options_merge_cache[$dealer_id])) {
            return $this->options_merge_cache[$dealer_id];
        }

        $conditions = '';

        if (isset($options['section_id'])) {
            $conditions .= ' AND section_id = ' . $options['section_id'];
        }

        return $this->options_merge_cache[$dealer_id] = $this->db->get_results(
            "SELECT * FROM {$this->db->prefix}4dealer_options_merge 
            WHERE 4dealer_price_id = " . (int)$dealer_id . $conditions,
            ARRAY_A
        );
    }

    function hide_product($hide_flag, $product)
    {
        if ($hide_flag && $product->get_catalog_visibility() == 'visible') {
            $product->set_catalog_visibility('hidden');
            $product->save();
        }
    }

    function get_products_info($data)
    {
        ini_set('max_execution_time', 1800);
        $this->logger("[$data->cmd_id] Задача загрузки карточек товаров запущена.");
        $hide_products_without_pics = get_option("{$this->plugin_name}-4dealer_hide_goods_without_pics", true);
        $_pf = new WC_Product_Factory();

        $j = 0;
        $fetch_product_info = Connector::getPriceItemInfoByIdArray(unserialize($data->data));

        if (isset($fetch_product_info['Result']) && empty($fetch_product_info['Error'])) {

            foreach ($fetch_product_info['Result'] as $product_info) {
                $this->logger("[$data->cmd_id] Карточка для {$product_info['name']}.");
                $product_id = wc_get_product_id_by_sku($product_info['id']);

                if (!$product_id) {
                    continue;
                }

                $product = $_pf->get_product($product_id);
                $j++;
                $options_string = '';

                $priceInfo = $this->getPriceInfoByCache($product_info['price_id']);

                if (isset($product_info['info'])) {
                    $options = json_decode($product_info['info']);


                    if ($options) {
                        $this->logger("[$data->cmd_id] Параметры найдены.");
                        $product = wc_get_product($product_id);
                        $product_local_cat = $product->get_category_ids()[0];

                        $dealer_cat = $this->get_dealer_category_by_local_cat($product_info['price_id'], $product_local_cat);

                        $table_options_enabled = $priceInfo['product_add_table'] == 1;

                        /** Фильтруем опции в соответствии с настройками прайса */
                        $options = $this->filterOptionsMerge($options, $product_info, $priceInfo, $product_local_cat);

                        if ($table_options_enabled) {
                            foreach ($options as $option) {
                                if (isset($option->name) && isset($option->value)) {
                                    $options_string .= $option->name . ": " . $option->value . "\n";
                                }
                            }
                        }

                        if (count($options)) {
                            $this->add_product_taxonomies($product, $product_info['price_id'], $product_id, $options, $product_local_cat, $data);
                        }
                    }
                    $this->logger("[$data->cmd_id] Параметры обработаны.");
                }

                $thumbnail_flag = has_post_thumbnail($product_id);

                if (isset($product_info['image']) && is_array($product_info['image']) && !$thumbnail_flag) {
                    $size = sizeof($product_info['image']);

                    if ($size > 0) {

                        if ($size > 1) {

                            $images_count = sizeof(explode(',', get_post_meta($product_id, '_product_image_gallery', true)));

                            if ($size - 1 > $images_count) {
                                $ids = array();

                                for ($i = 1; $i < $size; $i++) {
                                    $img_id = $this->save_image_with_url($product_info['image'][$i], $product_id, $data->cmd_id);
                                    if ($img_id) {
                                        $ids[] = $img_id;
                                    }
                                }

                                if (sizeof($ids)) {
                                    update_post_meta($product_id, '_product_image_gallery', implode(',', $ids));
                                }

                            }
                        }

                        $this->logger("[$data->cmd_id] Грузим картинки.");
                        $img_id = $this->save_image_with_url($product_info['image'][0], $product_id, $data->cmd_id);
                        $this->logger("[$data->cmd_id] Картинки загружены.");

                        if ($img_id) {
                            set_post_thumbnail($product_id, $img_id);
                            $thumbnail_flag = true;
                        }

                        if ($product->get_catalog_visibility() == 'hidden') {
                            $product->set_catalog_visibility('visible');
                            $product->save();
                        }

                    } else {
                        $this->logger("[$data->cmd_id] Скрываем продукт.");
                        $this->hide_product($hide_products_without_pics, $product);
                    }
                }

                if (!$thumbnail_flag) {
                    $this->hide_product($hide_products_without_pics, $product);
                    $this->logger("[$data->cmd_id] Скрываем продукт.");

                } else {
                    if ($product->get_catalog_visibility() == 'hidden') {
                        $product->set_catalog_visibility('visible');
                        $product->save();
                    }
                }

                wp_update_post(array('ID' => $product_id, 'post_content' => $product_info['description'] . "\n$options_string"));
            }


            $this->db->update("{$this->db->prefix}4dealer_cmd", array('status' => 2, 'records' => $j, 'end_time' => $this->get_current_mysql_time()), array('cmd_id' => $data->cmd_id));

            return true;
        }

        $this->db->update("{$this->db->prefix}4dealer_cmd", array('status' => 3, 'data' => serialize($fetch_product_info['Error']), 'records' => $j, 'end_time' => $this->get_current_mysql_time()), array('cmd_id' => $data->cmd_id));

        return false;
    }

    function filterOptionsMerge($options, $product_info, $priceInfo, $product_local_cat)
    {
        $result = [];

        $optionsMerge = $this->getOptionsMergeByCache($product_info['price_id'], $product_local_cat);

        if ($priceInfo['auto_load_options'] == '0') {

            // $this->logger("Пересекаем множество созданных свойств с переданными по Api .");
            // $this->logger("Прайс: {$product_info['price_id']}, секция {$dealer_cat}");
            // $this->logger("options: " . json_encode($options, JSON_UNESCAPED_UNICODE));
            // $this->logger("optionsMerge: " . json_encode($optionsMerge, JSON_UNESCAPED_UNICODE));
            // $this->logger("Пересекаем множество созданных свойств с переданными по Api .");

            /** Пересекаем множество созданных свойств ($optionsMerge) с переданными по Api ($options) */

            $options = (array) $options;

            foreach ($options as $option) {
                foreach ($optionsMerge as $optionMerge) {
                    $optionMerge = (array) $optionMerge;

                    if (trim($option->name) == trim($optionMerge['option_name'])) {
                        $result[] = $option;
                    }
                }
            }

        } else  {
            $result = $options;
        }

        return $result;
    }

    function getPriceInfoByCache($price_id)
    {
        $cacheKey = 'getCachePriceInfo:' . $price_id;

        if (isset($this->cache[$cacheKey])) {
            return (array) $this->cache[$cacheKey];
        }

        $result = $this->db->get_results(
            "SELECT * FROM {$this->db->prefix}4dealer_prices WHERE price_id = '{$price_id}'"
        );

        return $this->cache[$cacheKey] = (array) $result[0];
    }

    function getOptionsMergeByCache($price_id, $section_id)
    {
        $cacheKey = 'getOptionsMergeByCache:' . $price_id . ':' . $section_id;

        if (isset($this->cache[$cacheKey])) {
            return (array) $this->cache[$cacheKey];
        }

        $result = $this->db->get_results(
            "SELECT * FROM {$this->db->prefix}4dealer_options_merge 
            WHERE 4dealer_price_id = '{$price_id}' AND section_id = '{$section_id}'"
        );

        return $this->cache[$cacheKey] = (array) $result;
    }

    function get_dealer_category_by_local_cat($price_id, $localCatId)
    {
        $dealerCat = null;

        $dealer_categories = $this->get_categories_mapping_by_price_id($price_id, true);

        foreach ($dealer_categories as $dealer_cat => $local_category) {
            if ($localCatId == $local_category) {
                $dealerCat = $dealer_cat;
                break;
            }
        }

        return $dealerCat;
    }

    function save_image_with_url($url, $post_id, $cmd_id)
    {
        $this->logger("[$cmd_id] Картинка: {$url} .");
        $url = str_replace('http://', 'https://', $url);
        $tmp = download_url($url, 10);

        if (is_wp_error($tmp)) {
            $this->logger("[$cmd_id] Ошибка загрузки.");
            return false;
        }

        $this->logger("[$cmd_id] Временный файл загружен.");

        $file_array = array();

        @preg_match('/[^\?]+\.(jpg|jpe|jpeg|gif|png|pdf)/i', $url, $matches);

        if (!$matches) {
            $file_array['tmp_name'] = $tmp;
            $file_array['name'] = sanitize_file_name($url);
            $desc = '';
        } else {
            $file_array['name'] = basename($matches[0]);
            $file_array['tmp_name'] = $tmp;
            $desc = $file_array['name'];
        }

        $id = media_handle_sideload($file_array, $post_id, $desc);

        if (is_wp_error($id)) {
            $this->logger("[$cmd_id] Ошибка обработки загруженного файла.");
            @unlink($file_array['tmp_name']);
            return false;
        }

        $this->logger("[$cmd_id] Загружена [{$id}].");

        return $id;
    }

    function hide_duplicates($data)
    {
        @ini_set('max_execution_time', 720);
        $hide_option = get_option("{$this->plugin_name}-4dealer_hide_duplicates", true);
        if (!$hide_option) {
            return $this->set_command_result($data->cmd_id, 0, $data->price_id, false, 'hide_duplicates');
        }
        $hide_rule = get_option("{$this->plugin_name}-4dealer_hide_duplicates_rule", 1);
        $hide_products_without_pics = get_option("{$this->plugin_name}-4dealer_hide_goods_without_pics", true);

        $sort_type = $hide_rule == 1 ? SORT_DESC : SORT_ASC;

        $j = 0;

        $get_articuls = $this->db->get_results("SELECT name FROM {$this->db->prefix}terms, {$this->db->prefix}term_taxonomy where {$this->db->prefix}term_taxonomy.term_id={$this->db->prefix}terms.term_id and taxonomy='pa_part_no'; ");
        $_pf = new WC_Product_Factory();
        foreach ($get_articuls as $articul) {
            if ($articul->name == 'part_no') {
                continue;
            }

            $get_products_count = $this->db->get_results("SELECT count(post_id) as cnt FROM {$this->db->prefix}postmeta where meta_key='_product_attributes' and meta_value like '%{$articul->name}%';");
            if ($get_products_count[0]->cnt > 1) {

                $get_products = $this->db->get_results("SELECT post_id,meta_value FROM {$this->db->prefix}postmeta where meta_key='_product_attributes' and meta_value like '%{$articul->name}%';");
                $product_ids = array();
                $brands = array();
                foreach ($get_products as $product) {
                    $attributes = unserialize($product->meta_value);
                    if (!isset($attributes['pa_part_no']) || $attributes['pa_part_no']['value'] != $articul->name) {
                        //echo $attributes['pa_part_no']."\n";
                        continue;
                    }
                    $product_ids[] = $product->post_id;
                    if (!isset($brands[$attributes['pa_brand']['value']])) {
                        $brands[$attributes['pa_brand']['value']] = 1;
                    } else {
                        $brands[$attributes['pa_brand']['value']]++;
                    }
                }
                $product_objects = array();
                if (sizeof($product_ids) > 1 && sizeof($brands) == 1) {
                    $j++;
                    $price_and_avail_data = array();
                    foreach ($product_ids as $product_id) {
                        $product = $_pf->get_product($product_id);
                        if (!$product) {
                            continue;
                        }
                        $product_objects[$product_id] = $product;
                        $price_and_avail_data[$product_id] = array('quantity' => $product->get_stock_quantity(), 'price' => $product->get_price());
                        unset($product);
                    }
                    $this->sortBySubkey($price_and_avail_data, 'price', $sort_type);

                    $found_available = false;

                    foreach ($price_and_avail_data as $product_id => $product_data) {
                        if ($product_data["quantity"] > 0 && $product_data["price"] > 0) {
                            if (!$found_available) {
                                $found_available = true;
                                if ($product_objects[$product_id]->get_catalog_visibility() == 'hidden' && (!$hide_products_without_pics || has_post_thumbnail($product_id))) {
                                    $product_objects[$product_id]->set_catalog_visibility('visible');
                                    $product_objects[$product_id]->save();
                                }
                                continue;
                            }
                        }
                        if ($hide_option) {
                            $product_objects[$product_id]->set_catalog_visibility('hidden');
                            $product_objects[$product_id]->save();
                        }
                        //echo $articul->name."---".$product_objects[$product_id]->get_name()."---$product_id\n";
                    }
                    unset($product_objects, $price_and_avail_data);
                }
            }
        }
        return $this->set_command_result($data->cmd_id, $j, $data->price_id, false, 'hide_duplicates');
    }

    function sortBySubkey(&$array, $subkey, $sortType = SORT_ASC)
    {
        $all_keys = array_keys($array);

        array_multisort(array_column($array, $subkey), $sortType, SORT_NUMERIC, $array, $all_keys);
        $array = array_combine($all_keys, $array);
    }

    function logger($msg, $logfile = 'wp_fdealer')
    {
        if (get_option("{$this->plugin_name}-4dealer_logger", false)) {
            $fileName = WP_CONTENT_DIR . '/' . date('Y-m-d') . '_' . $logfile . '.log';
            $fp = fopen($fileName, 'a+');
            if (is_array($msg)) {
                fwrite($fp, date('Y-m-d H:i:s - ') . var_export($msg, true) . "\n");
            } else {
                fwrite($fp, date('Y-m-d H:i:s - ') . $msg . "\n");
            }
            fclose($fp);
        }
    }

    /**
     * Поиск подстроки с условиями ("содержит", "начинается на", "заканчивается на")
     * @param $string - строка, в которой осуществляется поиск
     * @param $substring - часть строки, которую ищем в $string
     * @param string $mode - режимы: "s" - начинается с $substring, "f" - наканчивается на $substring
     *                       по умолчанию не строгий - "содержит"
     * @param bool $caseSensitive - регистрозависимый, по умолчанию true
     * @return bool
     */
    function strIncludes($string, $substring, $mode = null, $caseSensitive = true)
    {
        if (!$caseSensitive) {
            $string = mb_strtolower($string);
            $substring = mb_strtolower($substring);
        }

        $pos = mb_strpos($string, $substring);

        if ($mode == 's') {
            $result = $pos === 0;
        } else if ($mode == 'f') {
            $result = $pos !== false && mb_strlen($string) === (mb_strlen($substring) + $pos);
        } else {
            $result = $pos !== false;
        }

        return $result;
    }
}
