<?php

class Connector
{
    const ERR_CODE_EMPTY_DATA = 103;
    const SOAP_ADDRESS = 'http://4dealer.ru/soap/?WSDL';
    private static $soap=false;
    private static $login;
    private static $key;
    public static $result;


    /**
     * Soap connect.
     * @return boolean|\SoapClient
     */
    private static function soap()
    {
        if (class_exists('SoapClient')) {
            if (!self::$soap) {
                self::$soap = new \SoapClient(
                        self::SOAP_ADDRESS,
                        [
                            'exceptions' => 0,
                            'cache_wsdl' => WSDL_CACHE_NONE,
                            "trace" => 1,
                            "stream_context" => stream_context_create(
                                [
                                    'ssl' => [
                                        'verify_peer'       => false,
                                        'verify_peer_name'  => false,
                                    ]
                                ]
                            )
                        ]
                );
                self::$login=get_option('wp-fdeal-4dealer_login');
                self::$key=get_option('wp-fdeal-4dealer_token');
                if(!self::$soap) return false;
            }
            return true;
        } else {
            return false;
        }
    }

    /**
     * Get the option.
     * @staticvar array $options
     * @param string|bool $key
     * @return string
     */
    private static function opt($key = false)
    {
        static $options = array();
        if (empty($options)) {
            $options = array(
                'login' => self::$login,
                'key' => self::$key,
            );
        }
        if ($key === false) {
            return $options;
        } else {
            return array_key_exists($key, $options) ? $options[$key] : false;
        }
    }

    /**
     * Get structure of catalog.
     * @param boolean $tree tree mode
     * @return array
     */
    public static function getCommonCatalogStructure($tree = true)
    {
        if ($client = self::soap()) {
            $result = (array)self::$soap->getCommonCatalogStructure(self::opt('login'), md5(self::opt('login') . 'getCommonCatalogStructure' . md5(self::opt('key'))), 'N');
            if (empty($result['Error'])) {
                return $tree ? self::buildTree($result['Result']) : $result['Result'];
            }
        }
        return array();
    }

    /**
     * Get structure for the price.
     * @param integer $price
     * @param boolean $tree tree mode
     * @return array
     */
    public static function getPriceStructure($price, $tree = true)
    {
        if ($client = self::soap()) {
            $result = (array)self::$soap->getPriceStructure(self::opt('login'), md5(self::opt('login') . 'getPriceStructure' . md5(self::opt('key'))), $price);
            if (empty($result['Error'])) {
                return $tree ? self::buildTree($result['Result']) : $result['Result'];
            }
        }
        return array();
    }

    /**
     * Get available prices for this account.
     * @return array
     */
    public static function getPricesList()
    {
        if ($client = self::soap()) {
            return (array)self::$soap->getPricesList(self::opt('login'), md5(self::opt('login') . 'getPricesList' . md5(self::opt('key'))), 'N');
        }
        return false;
    }

    /**
     * Get items for the $price.
     * @param integer $price
     * @param integer $offset
     * @param array $sections
     * @param integer $limit
     * @return array
     */
    public static function getPriceItems($price,$start_date=false,$last_update=false, $offset = 1, $sections = array(), $limit = 100)
    {
        if (self::soap()) {
            $params = array(
                //'only_with_articuls' => 'Y',
                'only_with_prices' => 'Y',
                'limit' => $limit,
                'offset' => $offset,
                //'avail' => 'all',
                'avail' => 'avail',
            );
            if (!empty($sections)) {
                $params['sections'] = $sections;
            }
            //$params['sections']=array(1443);
            if ($last_update) {
                $params['last_update'] = $last_update;
            }
            if ($start_date) {
                $params['start_date'] = $start_date;
            }
            //Wp_Fdeal_Admin::logger(array(self::$login, md5(self::$login . 'getPriceItems' . md5(self::$key)), $price, $params));
            return (array)self::$soap->getPriceItems(self::$login, md5(self::$login . 'getPriceItems' . md5(self::$key)), $price, $params);
        }
        return false;
    }

    public static function getPriceItemsByIdArray($targets){
        if(!is_array($targets)){
            return false;
        }

        if ($client = self::soap()) {
            $res = (array)self::$soap->getPriceItemByIdArray(self::opt('login'), md5(self::opt('login') . 'getPriceItemByIdArray' . md5(self::opt('key'))), $targets);
            return $res;

        }
        return false;

    }

    /**
     * Get items' info by array of id.
     * @param array $id
     * @return array
     */
    public static function getPriceItemInfoByIdArray($id)
    {
        if ($client = self::soap()) {
            return (array)self::$soap->getPriceItemInfoByIdArray(self::opt('login'), md5(self::opt('login') . 'getPriceItemInfoByIdArray' . md5(self::opt('key'))), $id);
        }
        return false;
    }

    /**
     * Get item's info by articul.
     * @param string $articul
     * @return array
     */
    public static function getPriceItemsByArticul($articul)
    {
        if ($client = self::soap()) {
            return (array)self::$soap->getPriceItemsByArticul(self::opt('login'), md5(self::opt('login') . 'getPriceItemsByArticul' . md5(self::opt('key'))), $articul);
        }
        return [];
    }

    /**
     * Возвращает доступные параметры для позиций указанных разделов каталога
     * дистрибьютора с их возможными значениями.
     * @param $sections
     * @return array
     */
    public static function getOptionsBySectionIdArray($sections = [])
    {
        if ($client = self::soap()) {
            return (array) self::$soap->getOptionsBySectionIdArray(
                self::opt('login'),
                md5(self::opt('login') . 'getOptionsBySectionIdArray' . md5(self::opt('key'))),
                (array) $sections,
                'N'
            );
        }

        return false;
    }

    /**
     * Show formatting error.
     * @param array $error
     */
    public static function showError($error)
    {
        if (!empty($error)) {
            var_dump($error);
        }
    }

    /**
     * Build flat array in the tree.
     * @param array &$elements
     * @param integer $parentId
     * @return array
     */
    private static function buildTree(&$elements, $parentId = 0,$parents=array())
    {
        $branch = array();
        foreach ($elements as $key => $element) {
            if ($element['parent'] != $parentId) {
                continue;
            }
            if(!in_array($parentId,$parents)){
                $parents[]=$parentId;
            }
            $element['parents']=$parents;
            $children = self::buildTree($elements, $element['id'],$parents);
            if (!empty($children)) {
                $element['children'] = $children;
            }
            unset($children);
            $branch[$element['id']] = $element;
        }
        unset($key, $element);
        $parents=array();
        return $branch;
    }
}

?>