<?php
class newConnector {

    const ERR_CODE_EMPTY_DATA = 103;
    const SOAP_ADDRESS = 'http://4dealer.ru/soap/?WSDL';
    private $soap;


    /**
     * Soap connect.
     * @return boolean|\SoapClient
     */
    public function soap()
    {
        if (class_exists('SoapClient')) {
            if(!$this->soap){
                $this->soap=new \SoapClient($this->SOAP_ADDRESS, array("trace" => 1, "exception" => 0));
            }
            return $this->soap;
        } else {
            return false;
        }
    }

    /**
     * Get the option.
     * @staticvar array $options
     * @param string|bool $key
     * @return string
     */
    public function opt($key = false)
    {
        static $options = array();
        if (empty($options)) {
            $options = array(
                'login' => get_option('wp-fdeal-4dealer_login'),
                'key' => get_option('wp-fdeal-4dealer_token'),
            );
        }
        if ($key === false) {
            return $options;
        } else {
            return array_key_exists($key, $options) ? $options[$key] : false;
        }
    }

    /**
     * Get structure of catalog.
     * @param boolean $tree tree mode
     * @return array
     */
    public function getCommonCatalogStructure($tree = true)
    {
        if ($client = $this->soap()) {
            $result = (array)$client->getCommonCatalogStructure($this->opt('login'), md5($this->opt('login') . 'getCommonCatalogStructure' . md5($this->opt('key'))), 'N');
            if (empty($result['Error'])) {
                return $tree ? $this->buildTree($result['Result']) : $result['Result'];
            }
        }
        return array();
    }

    /**
     * Get structure for the price.
     * @param integer $price
     * @param boolean $tree tree mode
     * @return array
     */
    public function getPriceStructure($price, $tree = true)
    {
        if ($client = $this->soap()) {
            $result = (array)$client->getPriceStructure($this->opt('login'), md5($this->opt('login') . 'getPriceStructure' . md5($this->opt('key'))), $price);
            if (empty($result['Error'])) {
                return $tree ? $this->buildTree($result['Result']) : $result['Result'];
            }
        }
        return array();
    }

    /**
     * Get available prices for this account.
     * @return array
     */
    public function getPricesList()
    {
        if ($client = $this->soap()) {
            return (array)$client->getPricesList($this->opt('login'), md5($this->opt('login') . 'getPricesList' . md5($this->opt('key'))), 'N');
        }
        return false;
    }

    /**
     * Get items for the $price.
     * @param integer $price
     * @param integer $offset
     * @param array $sections
     * @param integer $limit
     * @return array
     */
    public function getPriceItems($price,$lastUpdate='', $offset = 1, $sections = array(), $limit = 100)
    {
        if ($client = $this->soap()) {
            $params = array(
                //'only_with_articuls' => 'Y',
                'only_with_prices' => 'Y',
                'limit' => $limit,
                'offset' => $offset,
                //'avail' => 'all',
                'avail' => 'avail',
            );
            if (!empty($sections)) {
                $params['sections'] = $sections;
            }
            if (strlen($lastUpdate)) {
                $params['last_update'] = $lastUpdate;
            }

            $res=(array)$client->getPriceItems($this->opt('login'), md5($this->opt('login') . 'getPriceItems' . md5($this->opt('key'))), $price, $params);
            unset($client);

            return $res;

        }
        return false;
    }

    public function getPriceItemsByIdArray($targets){
        if(!is_array($targets)){
            return false;
        }

        if ($client = $this->soap()) {
            $res = (array)$client->getPriceItemByIdArray($this->opt('login'), md5($this->opt('login') . 'getPriceItemByIdArray' . md5($this->opt('key'))), $targets);
            return $res;

        }
        return false;

    }

    /**
     * Get items' info by array of id.
     * @param array $id
     * @return array
     */
    public function getPriceItemInfoByIdArray($id)
    {
        if ($client = $this->soap()) {
            return (array)$client->getPriceItemInfoByIdArray($this->opt('login'), md5($this->opt('login') . 'getPriceItemInfoByIdArray' . md5($this->opt('key'))), $id);
        }
        return false;
    }

    /**
     * Get item's info by articul.
     * @param string $articul
     * @return array
     */
    public function getPriceItemsByArticul($articul)
    {
        if ($client = $this->soap()) {
            return (array)$client->getPriceItemsByArticul($this->opt('login'), md5($this->opt('login') . 'getPriceItemsByArticul' . md5($this->opt('key'))), $articul);
        }
        return false;
    }

    /**
     * Show formatting error.
     * @param array $error
     */
    public function showError($error)
    {
        if (!empty($error)) {
            var_dump($error);
        }
    }

    /**
     * Build flat array in the tree.
     * @param array &$elements
     * @param integer $parentId
     * @return array
     */
    public function buildTree(&$elements, $parentId = 0,$parents=array())
    {
        $branch = array();
        foreach ($elements as $key => $element) {
            if ($element['parent'] != $parentId) {
                continue;
            }
            if(!in_array($parentId,$parents)){
                $parents[]=$parentId;
            }
            $element['parents']=$parents;
            $children = $this->buildTree($elements, $element['id'],$parents);
            if (!empty($children)) {
                $element['children'] = $children;
            }
            unset($children);
            $branch[$element['id']] = $element;
        }
        unset($key, $element);
        $parents=array();
        return $branch;
    }
    
    private function __construct(){
        
    }
    private function __clone(){
        
    }
    public function getInstance(){
        if($this->$__instance == NULL) $this->$__instance = new newConnector;
        return $this->$__instance;
    }
    
}
?>