<?php

/**
 * Fired during plugin activation
 *
 * @link       http://4dealer.ru/
 * @since      1.0.0
 *
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/includes
 * @author     Maksim Onokhin <root@cybermax.spb.ru>
 */
class Wp_Fdeal_Activator
{

    /**
     * Short Description. (use period)
     *
     * Long Description.
     *
     * @param $tableEngine
     * @since    1.0.0
     */
    public static function activate($tableEngine)
    {

        global $wp_version;
        global $wpdb;


        $plugin = plugin_basename(__FILE__);
        $plugin_data = get_plugin_data(__FILE__, false);

        if (version_compare($wp_version, "3.3", "<")) {
            if (is_plugin_active($plugin)) {
                deactivate_plugins($plugin);
                wp_die("'" . $plugin_data['Name'] . "' requires WordPress 3.3 or higher, and has been deactivated! Please upgrade WordPress and try again.<br /><br />Back to <a href='" . admin_url() . "'>WordPress Admin</a>.");
            }
        }

        if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            $table_name = $wpdb->prefix . "4dealer_prices";
            $charset_collate = $wpdb->get_charset_collate();

            $sql = "
            CREATE TABLE IF NOT EXISTS {$wpdb->prefix}4dealer_statuses (
            `status_id` tinyint(3) unsigned NOT NULL AUTO_INCREMENT,
            `status_name` varchar(100) NOT NULL,
            PRIMARY KEY (`status_id`)
            ) $charset_collate ENGINE=InnoDB;
            
            CREATE TABLE IF NOT EXISTS $table_name (
            `price_id` int(10) unsigned NOT NULL DEFAULT '0',
            `price_name` varchar(200)  NOT NULL,
            `local_last_update` datetime DEFAULT '1970-01-01 00:00:00',
            `local_last_full_update` datetime NOT NULL DEFAULT '1970-01-01 00:00:00',
            `4dealer_last_update` datetime DEFAULT '1970-01-01 00:00:00',
            `subscription_end` datetime NOT NULL,
            `categories` longtext,
            `active` tinyint(4) NOT NULL DEFAULT '1',
            `price_markup` smallint(5) unsigned DEFAULT '0',
            `small_quantity_transform` smallint(5) unsigned DEFAULT '1',
            `medium_quantity_transform` smallint(5) unsigned DEFAULT '10',
            `large_quantity_transform` smallint(5) unsigned DEFAULT '100',
            `schedule_update_time` tinyint(3) NOT NULL DEFAULT '-1',
            `round_price` int(10) NOT NULL,
            `round_rule` int(10) unsigned NOT NULL,
            `product_add_table` int(10) unsigned NOT NULL,
            `auto_load_options` int(10) unsigned NOT NULL,
            `public_options` INT(1) NOT NULL,
            PRIMARY KEY (`price_id`),
            KEY `local_update_idx` (`local_last_update`),
            KEY `remote_update_idx` (`4dealer_last_update`),
            KEY `local_full_update_idx` (`local_last_full_update`)
            ) $charset_collate ENGINE=InnoDB;                              
            
            CREATE TABLE IF NOT EXISTS {$wpdb->prefix}4dealer_categories_term (
            `term_id` bigint(20) unsigned NOT NULL,
            `4dealer_cat_id` int(11) unsigned NOT NULL,
            `4dealer_price_id` int(11) unsigned DEFAULT NULL,
            `is_active` tinyint(3) unsigned NOT NULL DEFAULT '1',
            PRIMARY KEY (`term_id`,`4dealer_cat_id`),
            KEY `price_id` (`4dealer_price_id`),
            KEY `active` (`is_active`)         
            ) $charset_collate ENGINE=InnoDB;
            
            CREATE TABLE IF NOT EXISTS {$wpdb->prefix}4dealer_price_markup (
            `id` INT NOT NULL AUTO_INCREMENT,
            `price_id` int(10) unsigned NOT NULL,
            `category_id` bigint(20) unsigned NOT NULL DEFAULT '0',
            `price_low` int(10) unsigned NOT NULL,
            `price_high` int(10) unsigned NOT NULL,
            `markup` int(10) unsigned NOT NULL,
            `price_is_rrc` int(10) unsigned NOT NULL,
            PRIMARY KEY (`id`)
            ) $charset_collate ENGINE=InnoDB;
            
            CREATE TABLE IF NOT EXISTS {$wpdb->prefix}4dealer_options_merge (
            `id` INT NOT NULL AUTO_INCREMENT,
            `4dealer_price_id` INT NOT NULL,
            `section_id` INT NOT NULL,
            `option_name` VARCHAR(255) NOT NULL,
            `changed_option_name` VARCHAR(255) NOT NULL,
            `replaces` TEXT NOT NULL,
            PRIMARY KEY (`id`),
            INDEX (`4dealer_price_id`)                    
            ) $charset_collate ENGINE=InnoDB;";

            if ($tableEngine === 'InnoDB') {
                $sql .= "
                ALTER TABLE {$wpdb->prefix}4dealer_categories_term ADD CONSTRAINT `{$wpdb->prefix}4dealer_categories_term_ibfk_1`
                FOREIGN KEY (`term_id`) REFERENCES `{$wpdb->prefix}terms` (`term_id`) ON DELETE CASCADE ON UPDATE CASCADE;";
            }

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            foreach (explode(';', $sql) as $query) {
                $query = trim($query);
                if (!$query) {
                    continue;
                }

                dbDelta($query);
            }

            $wpdb->insert("{$wpdb->prefix}4dealer_statuses", array('status_name' => 'Running'));
            $wpdb->insert("{$wpdb->prefix}4dealer_statuses", array('status_name' => 'Finished'));
            $wpdb->insert("{$wpdb->prefix}4dealer_statuses", array('status_name' => 'Error'));
            $wpdb->insert("{$wpdb->prefix}4dealer_statuses", array('status_name' => 'Waiting'));
            $wpdb->insert("{$wpdb->prefix}4dealer_statuses", array('status_name' => 'Canceled'));


            $create_cmd_table = "
            CREATE TABLE IF NOT EXISTS {$wpdb->prefix}4dealer_cmd (
            `cmd_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
            `parent_id` bigint(20) unsigned DEFAULT '0',
            `cmd` varchar(100) NOT NULL,
            `status` tinyint(3) unsigned NOT NULL DEFAULT '4',
            `create_time` datetime DEFAULT NULL,
            `start_time` datetime NOT NULL,
            `end_time` datetime DEFAULT '1970-01-01 00:00:00',
            `price_id` int(10) unsigned NOT NULL,
            `records` int(10) unsigned DEFAULT '0',
            `data` longtext COLLATE utf8_unicode_ci,
            `pid` int(10) unsigned DEFAULT NULL,
            `priority` tinyint(3) unsigned NOT NULL DEFAULT '50',
            PRIMARY KEY (`cmd_id`),
            KEY `time_idx` (`start_time`,`end_time`),
            KEY `cmd_idx` (`cmd`),
            KEY `fk_wp_4dealer_cmd_1_idx` (`price_id`),
            KEY `fk_wp_4dealer_cmd_2_idx` (`status`),
            KEY `parent_idx` (`parent_id`),
            KEY `priority_idx` (`priority`),
            CONSTRAINT `fk_wp_4dealer_cmd_1` FOREIGN KEY (`price_id`) REFERENCES `{$wpdb->prefix}4dealer_prices` (`price_id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
            CONSTRAINT `fk_wp_4dealer_cmd_2` FOREIGN KEY (`status`) REFERENCES `{$wpdb->prefix}4dealer_statuses` (`status_id`) ON DELETE NO ACTION ON UPDATE NO ACTION
            ) $charset_collate ENGINE=InnoDB;";

            dbDelta($create_cmd_table);

        } else {
            wp_die("'" . $plugin_data['Name'] . "' requires Woocommerce plugin installed, and has been deactivated! Please install Woocommerce and try again.<br /><br />Back to <a href='" . admin_url() . "'>WordPress Admin</a>.");
        }
    }

}
