<?php
if (!defined('WP_FDEAL_LOGGER')) {
    define('WP_FDEAL_LOGGER', true);
}


class dealerProduct
{
    protected $id;
    protected $post_id = false;
    protected $price_field = 'price_rub';
    protected $name;
    protected $no;
    protected $part_no;
    protected $brand_id;
    protected $brand;
    protected $price_category_id;
    protected $category_id;
    protected $price_id;
    protected $price_name;
    protected $avail;
    protected $free;
    protected $price_rub;
    protected $price_usd;
    protected $price_eur;
    protected $stock_status;
    protected $date_insert;
    protected $transit;
    protected $transit_next;
    protected $transit_next_data;
    protected $transit_long;
    protected $status;
    protected $action;
    protected $msrp;
    protected $msrp_r;
    protected $msrp_e;
    protected $date_updated;
    protected $small_quantity_transform;
    protected $medium_quantity_transform;
    protected $large_quantity_transform;
    protected $tags = array();
    public    $logfile = '';

    public    $priceInfo = array();


    protected $fieldsMapper = array(
        'price_id' => '_price_id',
    );

    protected $staticData = array('_visibility' => 'visible',
        'total_sales' => 0,
    );

    protected $attributesFields = array(
        'pa_no' => array(
            'name' => 'pa_no',
            'position' => 1,
            'is_visible' => 0,
            'is_variation' => 0,
            'is_taxonomy' => 1
        ),
        'pa_part_no' => array(
            'name' => 'pa_part_no',
            'position' => 2,
            'is_visible' => 1,
            'is_variation' => 0,
            'is_taxonomy' => 1
        ),
        'pa_brand' => array(
            'name' => 'pa_brand',
            'position' => 3,
            'is_visible' => 0,
            'is_variation' => 0,
            'is_taxonomy' => 1
        ),
    );

    protected $products_controller;
    protected $wp_rest_request;

    protected $attributesMapper = array('pa_no' => 'no', 'pa_part_no' => 'part_no','pa_brand'=>'brand');

    private $plugin_name;

    function __construct($data, $plugin_name, $priceInfo)
    {
        $this->plugin_name = $plugin_name;
        $this->priceInfo = $priceInfo;

        $this->logfile = WP_CONTENT_DIR . '/' . date('Y-m-d') . '_wp_fdealer.log';

        foreach ($data as $key => $value) {
            if (!is_array($value)) {
                $this->$key = $value;
            }
        }
        if (isset($data['my_prices'])) {
            foreach ($data['my_prices'] as $key => $value) {
                $this->$key = $value;
            }
        }
    }

    function preWork()
    {
        $this->free = $this->getQuantity();
        $this->stock_status = $this->free ? true : false;
    }

    function is_price_in_range($markup)
    {
        return $markup['price_low'] <= (float)$this->{$this->price_field} && (float)$this->{$this->price_field} <= $markup['price_high'];
    }

    /**
     * Определение цены с учетом наценки и/или флага "РРЦ как конечная цена"
     *
     * @param $markup
     * @param null $priceIsRrc - цена равна msrp_r
     * @return float|int|string
     */
    function calculateMarkup($markup, $priceIsRrc = false)
    {
        $this->logger('Тип цены: ' . print_r($this->price_field,true));
        $this->logger('Значение цены: '.print_r($this->{$this->price_field},true));
        $this->logger('Значение наценки:' . print_r($markup,true));
        $this->logger('РРЦ:' . print_r($this->msrp_r,true));

        /** Если стоит флаг $priceIsRrc, то возвращаем msrp_r */

        if ($priceIsRrc && is_numeric($this->msrp_r)) {
            $this->logger('Цена, используя РРЦ: ' . $this->msrp_r);

            return $this->msrp_r;
        } else {
            $price_markup = (float)$this->{$this->price_field} + round((float)$this->{$this->price_field} * (float)$markup / 100,2);
            $this->logger('Цена, используя правила наценки: ' . print_r($price_markup,true));

            return $price_markup;
        }

    }

    /**
     * Round price
     */
    function roundPrice()
    {
        $price = $this->{$this->price_field};

        /** Округление включено в настройках */
        if ($this->priceInfo['round_price'] != 0) {
            $roundPrice = $this->priceInfo['round_price'] ? $this->priceInfo['round_price'] : false;
            $roundRule = $this->priceInfo['round_rule'] == 0 ? PHP_ROUND_HALF_UP : PHP_ROUND_HALF_DOWN;

            // у округления до целого значение в select = 1, меняем на 0
            if ($roundPrice == 1 || !$roundPrice) {
                $roundPrice = 0;
            }

            if (round($price, $roundPrice, $roundRule) > 0) {
                $price = round($price, $roundPrice, $roundRule);
            } else {
                $price = round($price, 0, $roundRule);
            }

            $this->logger('Применено правило округления: ' . $roundPrice . ', ' . $roundRule);
        }

        $this->{$this->price_field} = $price;
    }

    function iterate_markups($markups)
    {
        foreach ($markups as $markup) {
            if ($this->is_price_in_range($markup)) {
                $this->{$this->price_field} = $this->calculateMarkup($markup['markup'], !!$markup['price_is_rrc']);
                return true;
            }
        }
        return false;
    }

    function recalculatePrice($general_markup, $categories_markup)
    {
        if ($general_markup || $categories_markup) {
            if (isset($categories_markup[$this->category_id])) {
                if ($this->iterate_markups($categories_markup[$this->category_id])) {
                    return true;
                }
            } elseif (isset($categories_markup[0])) {
                if ($this->iterate_markups($categories_markup[0])) {
                    return true;
                }
            }
            $this->{$this->price_field} = $this->calculateMarkup($general_markup);
            return true;

        }
        return false;
    }

    function getQuantity()
    {
        if (in_array(strtolower($this->free), array('', 'false', 'call', 'no', ' '))) {
            $newQuantity = 0;
        } else {
            switch ($this->free) {
                case '*':
                    $newQuantity = $this->small_quantity_transform;
                    break;
                case '**':
                    $newQuantity = $this->medium_quantity_transform;
                    break;
                case '***':
                    $newQuantity = $this->large_quantity_transform;
                    break;
                case 'Да':
                    $newQuantity = $this->large_quantity_transform;
                    break;
                default :
                    $newQuantity = preg_replace('~[^0-9]+~', '', $this->free);
                    break;
            }
        }

        if (empty($newQuantity))
            $newQuantity = 0;

        return $newQuantity;


    }

    function findMappings($mappings, $categories)
    {
        if (isset($mappings[$this->price_category_id])) {
            return $mappings[$this->price_category_id];
        }

        if (!is_array($categories[$this->price_category_id]['parents'])) {
            $this->logger('ERROR! Mappings not found:' . print_r($categories[$this->price_category_id],true));
            return false;
        }

        foreach (array_reverse($categories[$this->price_category_id]['parents']) as $parent_id) {
            if (isset($mappings[$parent_id])) {
                return $mappings[$parent_id];
            }
        }
        return false;

    }

    function update($need_attributes_update=false,$products_controller,$wp_put_rest_request,$fast_update=false,$db=false)
    {

        if(!$fast_update) {
            $rest_request = array('name' => $this->name, 'regular_price' => $this->{$this->price_field},
                'in_stock' => $this->stock_status, 'stock_quantity' => $this->free, 'id' => $this->post_id);
            $wp_put_rest_request->set_body_params($rest_request);
            $products_controller->update_item($wp_put_rest_request);
        }
        elseif($db){
            $stock_status = $this->stock_status ? 'instock' : 'outofstock';
            $db->query('SET autocommit = 0;');
            $db->query("UPDATE {$db->prefix}postmeta SET meta_value='$stock_status' where post_id={$this->post_id} and meta_key='_stock_status'");
            $db->query("UPDATE {$db->prefix}postmeta SET meta_value={$this->free} where post_id={$this->post_id} and meta_key='_stock'");
            $db->query("UPDATE {$db->prefix}postmeta SET meta_value=".(float)$this->{$this->price_field}." where post_id={$this->post_id} and meta_key='_price'");
            $db->query("UPDATE {$db->prefix}postmeta SET meta_value=".(float)$this->{$this->price_field}." where post_id={$this->post_id} and meta_key='_regular_price'");
            $db->query('COMMIT;');
            $db->query('SET autocommit = 1;');
            unset($stock_status);
        }

        if($need_attributes_update){
            if ($this->brand) {
                wp_set_object_terms($this->post_id, array($this->brand, $this->price_name), 'product_tag');
            }
            $this->updateAttributes();
        }
        return true;
    }

    function updateAttributes()
    {
        $product_attributes = array();

        foreach ($this->attributesFields as $attribute => $attr_details) {
            if ($this->{$this->attributesMapper[$attribute]}) {
                $attr_details['value'] = $this->{$this->attributesMapper[$attribute]};
                $product_attributes[$attr_details['name']] = $attr_details;
                wp_set_object_terms($this->post_id, $attr_details['value'], $attribute);
            }
        }

        if (sizeof($product_attributes)) {
            update_post_meta($this->post_id, '_product_attributes', $product_attributes);
        }

        $product_attributes=null;
    }


    function save($mappings, $categories,$products_controller,$wp_rest_request,$wp_put_rest_request,$variations_controler=false,$db=false)
    {
        $this->preWork();

        $category_id = $this->findMappings($mappings, $categories);

        if (!$category_id) {
            return false;
        }

        $rest_request=array('name'=>$this->name,'type'=>'simple','regular_price'=>$this->{$this->price_field},
            'categories' => array(array('id'=>$category_id)),'sku'=>$this->id, 'in_stock'=>$this->stock_status,'stock_quantity'=>$this->free);

        if (!$this->post_id) {
            $wp_rest_request->set_body_params( $rest_request );
            $res = $products_controller->create_item( $wp_rest_request );

            if(!$res){
                return false;
            }

            $res = $res->data;
            $this->post_id=$res['id'];

            $this->tags[] = $this->price_name;
            if ($this->brand) {
                $this->tags[] = $this->brand;
            }
            wp_set_object_terms($this->post_id, $this->tags, 'product_tag');

            foreach ($this->fieldsMapper as $key => $value) {
                add_post_meta($this->post_id, $value, $this->$key);
            }
            $this->updateAttributes();


            /*if(!$variations_controler){
                return true;
            }*/

            // The created product must have variations
            // If it doesn't, it's the new WC3+ API which forces us to build those manually
            if ( ! isset( $res['variations'] ) )
                $res['variations'] = array();
            if ( isset($res['variations']) && count( $res['variations'] ) == 0 && isset($rest_request['variations']) && count( $rest_request['variations'] ) > 0 ) {
                foreach ( $rest_request['variations'] as $variation ) {
                    $variation_rest = array(
                        'product_id' => $res['id'],
                        'regular_price' => $variation['regular_price'],
                        'image' => array( 'id' => $variation['image'][0]['id'], ),
                        'attributes' => $variation['attributes'],
                    );
                    $this->wp_rest_request->set_body_params( $variation_rest );
                    $new_variation = $variations_controler->create_item( $this->wp_rest_request );
                    $res['variations'][] = $new_variation->data;
                }
            }
            return true;
        }

        $this->update(true,$products_controller,$wp_put_rest_request,true,$db);
        unset($res);
        return true;


    }

    function logger($msg, $logfile = 'wp_fdealer')
    {
        if (get_option("{$this->plugin_name}-4dealer_logger", false)) {
            $fileName = WP_CONTENT_DIR . '/' . date('Y-m-d') . '_' . $logfile . '.log';
            $fp = fopen($fileName, 'a+');
            if (is_array($msg)) {
                fwrite($fp, date('Y-m-d H:i:s - ') . var_export($msg, true) . "\n");
            } else {
                fwrite($fp, date('Y-m-d H:i:s - ') . $msg . "\n");
            }
            fclose($fp);
        }
    }
}
