<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://4dealer.ru/
 * @since             1.0.0
 * @package           Wp_Fdeal
 *
 * @wordpress-plugin
 * Plugin Name:       4Dealer Importer • 4dealer.ru
 * Plugin URI:        https://4dealer.ru/api_wp
 * Description:       Официальный плагин WordPress для работы с 4dealer.ru
 * Version:           1.5.1
 * Author:            4dealer.ru
 * Author URI:        http://4dealer.ru/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wp-fdeal
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}




/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wp-fdeal-activator.php
 */
function activate_wp_fdeal() {
    global $wpdb;
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-fdeal-activator.php';

    $tableEngine = getTableEngine($wpdb->prefix . 'terms');

	Wp_Fdeal_Activator::activate($tableEngine);
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wp-fdeal-deactivator.php
 */
function deactivate_wp_fdeal() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wp-fdeal-deactivator.php';
	Wp_Fdeal_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wp_fdeal' );
register_deactivation_hook( __FILE__, 'deactivate_wp_fdeal' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wp-fdeal.php';

/**
 * The code that runs during plugin upgrading.
 */
function upgrade_wp_fdeal($upgrader_object, $hook_extra) {
    $plugin = new Wp_Fdeal();
    $plugin->runUpdateProcedure($upgrader_object->new_plugin_data['Version']);
}

add_action('upgrader_process_complete', 'upgrade_wp_fdeal', 10, 2);


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wp_fdeal() {
	$plugin = new Wp_Fdeal();
	$plugin->run();
}

function getTableEngine($table) {
    global $wpdb;

    $sql = "SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = '{$table}'";

    $res = $wpdb->get_results($sql);
    return $res[0]->ENGINE;
}

/**
 * Сортировка по алфавиту (с поддержкой кириллицы)
 *
 * @param $a
 * @param $b
 * @return int
 */
function mb_strcmp($a, $b) {
    $pattern = '/[А-Яа-яЁё]/u'; // кириллица

    if (!preg_match($pattern, $a) && !preg_match($pattern, $a)) {
        return strcmp($a, $b);
    }

    $la = mb_substr($a, 0, 1, 'utf-8');
    $lb = mb_substr($b, 0, 1, 'utf-8');

    if (ord($la) > 122 && ord($lb) > 122) {
        return $a > $b ? 1 : -1;
    }

    if (ord($la) > 122 || ord($lb) > 122) {
        return $a < $b ? 1 : -1;
    }
}

run_wp_fdeal();
